/*
  GUIDO Library
  Copyright (C) 2014-2019	Grame

  This Source Code Form is subject to the terms of the Mozilla Public
  License, v. 2.0. If a copy of the MPL was not distributed with this
  file, You can obtain one at http://mozilla.org/MPL/2.0/.

  Grame Research Laboratory, 11, cours de Verdun Gensoul 69002 Lyon - France
  research@grame.fr

*/

#include <iostream>
#include "MusicFont.h"

using namespace std;

#ifdef SMUFL

void MusicFont::initialize()
{
	fRefSize = kFontSize;			// the font size used to compute the extends map
	fHeight	 = kBravuraFontHeight;	// the static height value
	fTblOffset = 0xE000;

	fExtends[0] = 17;
	fExtends[1] = 19;
	fExtends[2] = 112;
	fExtends[3] = 94;
	fExtends[4] = 94;
	fExtends[5] = 94;
	fExtends[6] = 94;
	fExtends[7] = 212;
	fExtends[8] = 312;
	fExtends[9] = 464;
	fExtends[10] = 60;
	fExtends[11] = 105;
	fExtends[12] = 105;
	fExtends[13] = 105;
	fExtends[14] = 59;
	fExtends[15] = 59;
	fExtends[16] = 100;
	fExtends[17] = 100;
	fExtends[18] = 100;
	fExtends[19] = 100;
	fExtends[20] = 100;
	fExtends[21] = 100;
	fExtends[22] = 150;
	fExtends[23] = 150;
	fExtends[24] = 150;
	fExtends[25] = 150;
	fExtends[26] = 150;
	fExtends[27] = 150;
	fExtends[28] = 50;
	fExtends[29] = 50;
	fExtends[30] = 50;
	fExtends[31] = 50;
	fExtends[32] = 50;
	fExtends[33] = 50;
	fExtends[34] = 66;
	fExtends[35] = 106;
	fExtends[36] = 26;
	fExtends[37] = 59;
	fExtends[38] = 59;
	fExtends[39] = 59;
	fExtends[40] = 59;
	fExtends[41] = 59;
	fExtends[42] = 59;
	fExtends[43] = 59;
	fExtends[44] = 59;
	fExtends[45] = 59;
	fExtends[46] = 59;
	fExtends[47] = 59;
	fExtends[48] = 7;
	fExtends[49] = 29;
	fExtends[50] = 46;
	fExtends[51] = 46;
	fExtends[52] = 25;
	fExtends[53] = 69;
	fExtends[54] = 7;
	fExtends[55] = 10;
	fExtends[56] = 7;
	fExtends[57] = 7;
	fExtends[58] = 59;
	fExtends[59] = 59;
	fExtends[60] = 59;
	fExtends[61] = 59;
	fExtends[62] = 59;
	fExtends[63] = 59;
	fExtends[64] = 74;
	fExtends[65] = 73;
	fExtends[66] = 122;
	fExtends[67] = 20;
	fExtends[68] = 20;
	fExtends[69] = 216;
	fExtends[70] = 217;
	fExtends[71] = 111;
	fExtends[72] = 191;
	fExtends[73] = 164;
	fExtends[74] = 80;
	fExtends[75] = 80;
	fExtends[76] = 74;
	fExtends[77] = 74;
	fExtends[78] = 59;
	fExtends[79] = 59;
	fExtends[80] = 134;
	fExtends[81] = 134;
	fExtends[82] = 133;
	fExtends[83] = 134;
	fExtends[84] = 134;
	fExtends[85] = 209;
	fExtends[86] = 168;
	fExtends[87] = 133;
	fExtends[88] = 138;
	fExtends[89] = 134;
	fExtends[90] = 134;
	fExtends[91] = 135;
	fExtends[92] = 140;
	fExtends[93] = 140;
	fExtends[94] = 140;
	fExtends[95] = 140;
	fExtends[96] = 117;
	fExtends[97] = 47;
	fExtends[98] = 137;
	fExtends[99] = 137;
	fExtends[100] = 137;
	fExtends[101] = 137;
	fExtends[102] = 137;
	fExtends[103] = 137;
	fExtends[104] = 136;
	fExtends[105] = 76;
	fExtends[106] = 62;
	fExtends[107] = 50;
	fExtends[108] = 93;
	fExtends[109] = 82;
	fExtends[110] = 54;
	fExtends[111] = 62;
	fExtends[112] = 62;
	fExtends[113] = 124;
	fExtends[114] = 123;
	fExtends[115] = 135;
	fExtends[116] = 135;
	fExtends[117] = 140;
	fExtends[118] = 138;
	fExtends[119] = 137;
	fExtends[120] = 172;
	fExtends[121] = 57;
	fExtends[122] = 88;
	fExtends[123] = 101;
	fExtends[124] = 92;
	fExtends[125] = 41;
	fExtends[126] = 72;
	fExtends[127] = 0;
	fExtends[128] = 94;
	fExtends[129] = 67;
	fExtends[130] = 89;
	fExtends[131] = 84;
	fExtends[132] = 94;
	fExtends[133] = 81;
	fExtends[134] = 87;
	fExtends[135] = 88;
	fExtends[136] = 87;
	fExtends[137] = 87;
	fExtends[138] = 85;
	fExtends[139] = 83;
	fExtends[140] = 100;
	fExtends[141] = 51;
	fExtends[142] = 76;
	fExtends[143] = 90;
	fExtends[144] = 90;
	fExtends[145] = 79;
	fExtends[146] = 22;
	fExtends[147] = 22;
	fExtends[148] = 37;
	fExtends[149] = 37;
	fExtends[150] = 40;
	fExtends[151] = 83;
	fExtends[152] = 85;
	fExtends[153] = 90;
	fExtends[154] = 88;
	fExtends[155] = 89;
	fExtends[156] = 81;
	fExtends[157] = 169;
	fExtends[158] = 0;
	fExtends[159] = 0;
	fExtends[160] = 120;
	fExtends[161] = 83;
	fExtends[162] = 84;
	fExtends[163] = 59;
	fExtends[164] = 59;
	fExtends[165] = 59;
	fExtends[166] = 109;
	fExtends[167] = 75;
	fExtends[168] = 67;
	fExtends[169] = 58;
	fExtends[170] = 50;
	fExtends[171] = 50;
	fExtends[172] = 95;
	fExtends[173] = 57;
	fExtends[174] = 52;
	fExtends[175] = 50;
	fExtends[176] = 84;
	fExtends[177] = 50;
	fExtends[178] = 50;
	fExtends[179] = 50;
	fExtends[180] = 119;
	fExtends[181] = 84;
	fExtends[182] = 59;
	fExtends[183] = 58;
	fExtends[184] = 63;
	fExtends[185] = 63;
	fExtends[186] = 97;
	fExtends[187] = 64;
	fExtends[188] = 57;
	fExtends[189] = 59;
	fExtends[190] = 59;
	fExtends[191] = 68;
	fExtends[192] = 68;
	fExtends[193] = 68;
	fExtends[194] = 68;
	fExtends[195] = 97;
	fExtends[196] = 64;
	fExtends[197] = 57;
	fExtends[198] = 59;
	fExtends[199] = 58;
	fExtends[200] = 68;
	fExtends[201] = 68;
	fExtends[202] = 63;
	fExtends[203] = 63;
	fExtends[204] = 56;
	fExtends[205] = 56;
	fExtends[206] = 59;
	fExtends[207] = 59;
	fExtends[208] = 59;
	fExtends[209] = 58;
	fExtends[210] = 59;
	fExtends[211] = 84;
	fExtends[212] = 84;
	fExtends[213] = 119;
	fExtends[214] = 119;
	fExtends[215] = 86;
	fExtends[216] = 54;
	fExtends[217] = 50;
	fExtends[218] = 70;
	fExtends[219] = 50;
	fExtends[220] = 70;
	fExtends[221] = 50;
	fExtends[222] = 70;
	fExtends[223] = 90;
	fExtends[224] = 59;
	fExtends[225] = 58;
	fExtends[226] = 58;
	fExtends[227] = 58;
	fExtends[228] = 59;
	fExtends[229] = 59;
	fExtends[230] = 87;
	fExtends[231] = 121;
	fExtends[232] = 59;
	fExtends[233] = 59;
	fExtends[234] = 108;
	fExtends[235] = 111;
	fExtends[236] = 91;
	fExtends[237] = 110;
	fExtends[238] = 72;
	fExtends[239] = 72;
	fExtends[240] = 66;
	fExtends[241] = 110;
	fExtends[242] = 72;
	fExtends[243] = 72;
	fExtends[244] = 66;
	fExtends[245] = 15;
	fExtends[246] = 15;
	fExtends[247] = 50;
	fExtends[248] = 77;
	fExtends[249] = 77;
	fExtends[250] = 84;
	fExtends[251] = 59;
	fExtends[252] = 58;
	fExtends[253] = 59;
	fExtends[254] = 59;
	fExtends[255] = 59;
	fExtends[256] = 73;
	fExtends[257] = 106;
	fExtends[258] = 196;
	fExtends[259] = 156;
	fExtends[260] = 100;
	fExtends[261] = 37;
	fExtends[262] = 106;
	fExtends[263] = 73;
	fExtends[264] = 106;
	fExtends[265] = 156;
	fExtends[266] = 232;
	fExtends[267] = 59;
	fExtends[268] = 59;
	fExtends[269] = 59;
	fExtends[270] = 59;
	fExtends[271] = 59;
	fExtends[272] = 100;
	fExtends[273] = 100;
	fExtends[274] = 100;
	fExtends[275] = 50;
	fExtends[276] = 50;
	fExtends[277] = 50;
	fExtends[278] = 100;
	fExtends[279] = 100;
	fExtends[280] = 50;
	fExtends[281] = 50;
	fExtends[282] = 100;
	fExtends[283] = 100;
	fExtends[284] = 50;
	fExtends[285] = 50;
	fExtends[286] = 59;
	fExtends[287] = 59;
	fExtends[288] = 66;
	fExtends[289] = 66;
	fExtends[290] = 75;
	fExtends[291] = 66;
	fExtends[292] = 121;
	fExtends[293] = 86;
	fExtends[294] = 62;
	fExtends[295] = 65;
	fExtends[296] = 121;
	fExtends[297] = 85;
	fExtends[298] = 63;
	fExtends[299] = 72;
	fExtends[300] = 120;
	fExtends[301] = 120;
	fExtends[302] = 120;
	fExtends[303] = 85;
	fExtends[304] = 85;
	fExtends[305] = 85;
	fExtends[306] = 60;
	fExtends[307] = 60;
	fExtends[308] = 60;
	fExtends[309] = 59;
	fExtends[310] = 59;
	fExtends[311] = 59;
	fExtends[312] = 53;
	fExtends[313] = 53;
	fExtends[314] = 55;
	fExtends[315] = 55;
	fExtends[316] = 58;
	fExtends[317] = 58;
	fExtends[318] = 58;
	fExtends[319] = 59;
	fExtends[320] = 59;
	fExtends[321] = 59;
	fExtends[322] = 69;
	fExtends[323] = 69;
	fExtends[324] = 69;
	fExtends[325] = 69;
	fExtends[326] = 69;
	fExtends[327] = 69;
	fExtends[328] = 59;
	fExtends[329] = 59;
	fExtends[330] = 59;
	fExtends[331] = 59;
	fExtends[332] = 59;
	fExtends[333] = 59;
	fExtends[334] = 59;
	fExtends[335] = 59;
	fExtends[336] = 97;
	fExtends[337] = 97;
	fExtends[338] = 97;
	fExtends[339] = 97;
	fExtends[340] = 97;
	fExtends[341] = 97;
	fExtends[342] = 97;
	fExtends[343] = 97;
	fExtends[344] = 72;
	fExtends[345] = 72;
	fExtends[346] = 72;
	fExtends[347] = 72;
	fExtends[348] = 72;
	fExtends[349] = 72;
	fExtends[350] = 72;
	fExtends[351] = 72;
	fExtends[352] = 72;
	fExtends[353] = 72;
	fExtends[354] = 72;
	fExtends[355] = 72;
	fExtends[356] = 72;
	fExtends[357] = 72;
	fExtends[358] = 72;
	fExtends[359] = 72;
	fExtends[360] = 97;
	fExtends[361] = 97;
	fExtends[362] = 97;
	fExtends[363] = 97;
	fExtends[364] = 97;
	fExtends[365] = 97;
	fExtends[366] = 97;
	fExtends[367] = 97;
	fExtends[368] = 97;
	fExtends[369] = 97;
	fExtends[370] = 97;
	fExtends[371] = 97;
	fExtends[372] = 97;
	fExtends[373] = 97;
	fExtends[374] = 97;
	fExtends[375] = 97;
	fExtends[376] = 97;
	fExtends[377] = 97;
	fExtends[378] = 97;
	fExtends[379] = 97;
	fExtends[380] = 97;
	fExtends[381] = 97;
	fExtends[382] = 97;
	fExtends[383] = 72;
	fExtends[384] = 72;
	fExtends[385] = 72;
	fExtends[386] = 72;
	fExtends[387] = 72;
	fExtends[388] = 72;
	fExtends[389] = 72;
	fExtends[390] = 72;
	fExtends[391] = 72;
	fExtends[392] = 72;
	fExtends[393] = 72;
	fExtends[394] = 72;
	fExtends[395] = 72;
	fExtends[396] = 72;
	fExtends[397] = 72;
	fExtends[398] = 72;
	fExtends[399] = 72;
	fExtends[400] = 72;
	fExtends[401] = 72;
	fExtends[402] = 72;
	fExtends[403] = 72;
	fExtends[404] = 72;
	fExtends[405] = 72;
	fExtends[406] = 72;
	fExtends[407] = 72;
	fExtends[408] = 72;
	fExtends[409] = 71;
	fExtends[410] = 71;
	fExtends[411] = 71;
	fExtends[412] = 71;
	fExtends[413] = 72;
	fExtends[414] = 71;
	fExtends[415] = 71;
	fExtends[416] = 71;
	fExtends[417] = 71;
	fExtends[418] = 71;
	fExtends[419] = 72;
	fExtends[420] = 71;
	fExtends[421] = 71;
	fExtends[422] = 72;
	fExtends[423] = 71;
	fExtends[424] = 72;
	fExtends[425] = 72;
	fExtends[426] = 71;
	fExtends[427] = 72;
	fExtends[428] = 72;
	fExtends[429] = 97;
	fExtends[430] = 72;
	fExtends[431] = 72;
	fExtends[432] = 73;
	fExtends[433] = 72;
	fExtends[434] = 72;
	fExtends[435] = 72;
	fExtends[436] = 72;
	fExtends[437] = 72;
	fExtends[438] = 72;
	fExtends[439] = 72;
	fExtends[440] = 72;
	fExtends[441] = 72;
	fExtends[442] = 71;
	fExtends[443] = 71;
	fExtends[444] = 72;
	fExtends[445] = 72;
	fExtends[446] = 71;
	fExtends[447] = 71;
	fExtends[448] = 59;
	fExtends[449] = 59;
	fExtends[450] = 52;
	fExtends[451] = 52;
	fExtends[452] = 56;
	fExtends[453] = 56;
	fExtends[454] = 52;
	fExtends[455] = 52;
	fExtends[456] = 56;
	fExtends[457] = 56;
	fExtends[458] = 65;
	fExtends[459] = 65;
	fExtends[460] = 59;
	fExtends[461] = 59;
	fExtends[462] = 59;
	fExtends[463] = 59;
	fExtends[464] = 131;
	fExtends[465] = 107;
	fExtends[466] = 92;
	fExtends[467] = 68;
	fExtends[468] = 68;
	fExtends[469] = 66;
	fExtends[470] = 66;
	fExtends[471] = 113;
	fExtends[472] = 66;
	fExtends[473] = 116;
	fExtends[474] = 66;
	fExtends[475] = 113;
	fExtends[476] = 66;
	fExtends[477] = 113;
	fExtends[478] = 66;
	fExtends[479] = 113;
	fExtends[480] = 66;
	fExtends[481] = 113;
	fExtends[482] = 66;
	fExtends[483] = 114;
	fExtends[484] = 66;
	fExtends[485] = 113;
	fExtends[486] = 66;
	fExtends[487] = 20;
	fExtends[488] = 59;
	fExtends[489] = 59;
	fExtends[490] = 59;
	fExtends[491] = 59;
	fExtends[492] = 59;
	fExtends[493] = 59;
	fExtends[494] = 59;
	fExtends[495] = 59;
	fExtends[496] = 66;
	fExtends[497] = 66;
	fExtends[498] = 68;
	fExtends[499] = 68;
	fExtends[500] = 68;
	fExtends[501] = 68;
	fExtends[502] = 68;
	fExtends[503] = 68;
	fExtends[504] = 68;
	fExtends[505] = 68;
	fExtends[506] = 68;
	fExtends[507] = 68;
	fExtends[508] = 20;
	fExtends[509] = 73;
	fExtends[510] = 64;
	fExtends[511] = 47;
	fExtends[512] = 64;
	fExtends[513] = 64;
	fExtends[514] = 47;
	fExtends[515] = 64;
	fExtends[516] = 59;
	fExtends[517] = 59;
	fExtends[518] = 59;
	fExtends[519] = 59;
	fExtends[520] = 59;
	fExtends[521] = 59;
	fExtends[522] = 59;
	fExtends[523] = 59;
	fExtends[524] = 59;
	fExtends[525] = 59;
	fExtends[526] = 59;
	fExtends[527] = 59;
	fExtends[528] = 3;
	fExtends[529] = 3;
	fExtends[530] = 3;
	fExtends[531] = 3;
	fExtends[532] = 3;
	fExtends[533] = 3;
	fExtends[534] = 3;
	fExtends[535] = 3;
	fExtends[536] = 3;
	fExtends[537] = 3;
	fExtends[538] = 3;
	fExtends[539] = 3;
	fExtends[540] = 3;
	fExtends[541] = 3;
	fExtends[542] = 3;
	fExtends[543] = 3;
	fExtends[544] = 30;
	fExtends[545] = 30;
	fExtends[546] = 30;
	fExtends[547] = 30;
	fExtends[548] = 30;
	fExtends[549] = 30;
	fExtends[550] = 30;
	fExtends[551] = 30;
	fExtends[552] = 30;
	fExtends[553] = 30;
	fExtends[554] = 31;
	fExtends[555] = 34;
	fExtends[556] = 32;
	fExtends[557] = 38;
	fExtends[558] = 50;
	fExtends[559] = 80;
	fExtends[560] = 110;
	fExtends[561] = 80;
	fExtends[562] = 29;
	fExtends[563] = 54;
	fExtends[564] = 26;
	fExtends[565] = 59;
	fExtends[566] = 59;
	fExtends[567] = 59;
	fExtends[568] = 59;
	fExtends[569] = 59;
	fExtends[570] = 59;
	fExtends[571] = 59;
	fExtends[572] = 59;
	fExtends[573] = 59;
	fExtends[574] = 59;
	fExtends[575] = 59;
	fExtends[576] = 53;
	fExtends[577] = 61;
	fExtends[578] = 56;
	fExtends[579] = 58;
	fExtends[580] = 52;
	fExtends[581] = 55;
	fExtends[582] = 52;
	fExtends[583] = 55;
	fExtends[584] = 52;
	fExtends[585] = 55;
	fExtends[586] = 54;
	fExtends[587] = 60;
	fExtends[588] = 54;
	fExtends[589] = 60;
	fExtends[590] = 54;
	fExtends[591] = 62;
	fExtends[592] = 53;
	fExtends[593] = 60;
	fExtends[594] = 59;
	fExtends[595] = 59;
	fExtends[596] = 59;
	fExtends[597] = 59;
	fExtends[598] = 59;
	fExtends[599] = 59;
	fExtends[600] = 59;
	fExtends[601] = 59;
	fExtends[602] = 59;
	fExtends[603] = 59;
	fExtends[604] = 59;
	fExtends[605] = 59;
	fExtends[606] = 59;
	fExtends[607] = 59;
	fExtends[608] = 45;
	fExtends[609] = 34;
	fExtends[610] = 50;
	fExtends[611] = 50;
	fExtends[612] = 83;
	fExtends[613] = 103;
	fExtends[614] = 120;
	fExtends[615] = 92;
	fExtends[616] = 96;
	fExtends[617] = 104;
	fExtends[618] = 28;
	fExtends[619] = 28;
	fExtends[620] = 15;
	fExtends[621] = 15;
	fExtends[622] = 59;
	fExtends[623] = 59;
	fExtends[624] = 50;
	fExtends[625] = 50;
	fExtends[626] = 42;
	fExtends[627] = 34;
	fExtends[628] = 50;
	fExtends[629] = 50;
	fExtends[630] = 50;
	fExtends[631] = 50;
	fExtends[632] = 82;
	fExtends[633] = 83;
	fExtends[634] = 37;
	fExtends[635] = 37;
	fExtends[636] = 59;
	fExtends[637] = 59;
	fExtends[638] = 59;
	fExtends[639] = 59;
	fExtends[640] = 45;
	fExtends[641] = 93;
	fExtends[642] = 36;
	fExtends[643] = 63;
	fExtends[644] = 33;
	fExtends[645] = 83;
	fExtends[646] = 59;
	fExtends[647] = 59;
	fExtends[648] = 59;
	fExtends[649] = 59;
	fExtends[650] = 59;
	fExtends[651] = 59;
	fExtends[652] = 59;
	fExtends[653] = 59;
	fExtends[654] = 59;
	fExtends[655] = 59;
	fExtends[656] = 62;
	fExtends[657] = 61;
	fExtends[658] = 62;
	fExtends[659] = 61;
	fExtends[660] = 77;
	fExtends[661] = 77;
	fExtends[662] = 73;
	fExtends[663] = 73;
	fExtends[664] = 73;
	fExtends[665] = 46;
	fExtends[666] = 46;
	fExtends[667] = 74;
	fExtends[668] = 63;
	fExtends[669] = 59;
	fExtends[670] = 59;
	fExtends[671] = 59;
	fExtends[672] = 61;
	fExtends[673] = 54;
	fExtends[674] = 64;
	fExtends[675] = 61;
	fExtends[676] = 54;
	fExtends[677] = 54;
	fExtends[678] = 59;
	fExtends[679] = 59;
	fExtends[680] = 59;
	fExtends[681] = 59;
	fExtends[682] = 59;
	fExtends[683] = 59;
	fExtends[684] = 59;
	fExtends[685] = 59;
	fExtends[686] = 59;
	fExtends[687] = 59;
	fExtends[688] = 30;
	fExtends[689] = 30;
	fExtends[690] = 32;
	fExtends[691] = 32;
	fExtends[692] = 36;
	fExtends[693] = 36;
	fExtends[694] = 40;
	fExtends[695] = 40;
	fExtends[696] = 59;
	fExtends[697] = 59;
	fExtends[698] = 59;
	fExtends[699] = 59;
	fExtends[700] = 59;
	fExtends[701] = 59;
	fExtends[702] = 59;
	fExtends[703] = 59;
	fExtends[704] = 84;
	fExtends[705] = 46;
	fExtends[706] = 34;
	fExtends[707] = 50;
	fExtends[708] = 49;
	fExtends[709] = 83;
	fExtends[710] = 45;
	fExtends[711] = 34;
	fExtends[712] = 50;
	fExtends[713] = 50;
	fExtends[714] = 84;
	fExtends[715] = 46;
	fExtends[716] = 34;
	fExtends[717] = 50;
	fExtends[718] = 49;
	fExtends[719] = 83;
	fExtends[720] = 45;
	fExtends[721] = 34;
	fExtends[722] = 50;
	fExtends[723] = 50;
	fExtends[724] = 84;
	fExtends[725] = 46;
	fExtends[726] = 34;
	fExtends[727] = 50;
	fExtends[728] = 49;
	fExtends[729] = 83;
	fExtends[730] = 45;
	fExtends[731] = 34;
	fExtends[732] = 50;
	fExtends[733] = 49;
	fExtends[734] = 34;
	fExtends[735] = 34;
	fExtends[736] = 34;
	fExtends[737] = 34;
	fExtends[738] = 45;
	fExtends[739] = 54;
	fExtends[740] = 45;
	fExtends[741] = 54;
	fExtends[742] = 40;
	fExtends[743] = 40;
	fExtends[744] = 40;
	fExtends[745] = 40;
	fExtends[746] = 49;
	fExtends[747] = 49;
	fExtends[748] = 59;
	fExtends[749] = 59;
	fExtends[750] = 28;
	fExtends[751] = 28;
	fExtends[752] = 83;
	fExtends[753] = 45;
	fExtends[754] = 34;
	fExtends[755] = 50;
	fExtends[756] = 49;
	fExtends[757] = 45;
	fExtends[758] = 54;
	fExtends[759] = 40;
	fExtends[760] = 40;
	fExtends[761] = 87;
	fExtends[762] = 87;
	fExtends[763] = 90;
	fExtends[764] = 59;
	fExtends[765] = 59;
	fExtends[766] = 59;
	fExtends[767] = 59;
	fExtends[768] = 31;
	fExtends[769] = 31;
	fExtends[770] = 31;
	fExtends[771] = 31;
	fExtends[772] = 31;
	fExtends[773] = 31;
	fExtends[774] = 50;
	fExtends[775] = 50;
	fExtends[776] = 56;
	fExtends[777] = 56;
	fExtends[778] = 69;
	fExtends[779] = 69;
	fExtends[780] = 69;
	fExtends[781] = 69;
	fExtends[782] = 75;
	fExtends[783] = 75;
	fExtends[784] = 69;
	fExtends[785] = 69;
	fExtends[786] = 50;
	fExtends[787] = 50;
	fExtends[788] = 56;
	fExtends[789] = 56;
	fExtends[790] = 69;
	fExtends[791] = 69;
	fExtends[792] = 81;
	fExtends[793] = 81;
	fExtends[794] = 59;
	fExtends[795] = 59;
	fExtends[796] = 62;
	fExtends[797] = 62;
	fExtends[798] = 62;
	fExtends[799] = 62;
	fExtends[800] = 62;
	fExtends[801] = 62;
	fExtends[802] = 75;
	fExtends[803] = 75;
	fExtends[804] = 81;
	fExtends[805] = 81;
	fExtends[806] = 94;
	fExtends[807] = 94;
	fExtends[808] = 94;
	fExtends[809] = 94;
	fExtends[810] = 100;
	fExtends[811] = 100;
	fExtends[812] = 81;
	fExtends[813] = 81;
	fExtends[814] = 75;
	fExtends[815] = 75;
	fExtends[816] = 81;
	fExtends[817] = 81;
	fExtends[818] = 81;
	fExtends[819] = 81;
	fExtends[820] = 94;
	fExtends[821] = 94;
	fExtends[822] = 59;
	fExtends[823] = 59;
	fExtends[824] = 59;
	fExtends[825] = 59;
	fExtends[826] = 59;
	fExtends[827] = 59;
	fExtends[828] = 59;
	fExtends[829] = 59;
	fExtends[830] = 59;
	fExtends[831] = 59;
	fExtends[832] = 44;
	fExtends[833] = 44;
	fExtends[834] = 56;
	fExtends[835] = 56;
	fExtends[836] = 44;
	fExtends[837] = 44;
	fExtends[838] = 37;
	fExtends[839] = 37;
	fExtends[840] = 62;
	fExtends[841] = 63;
	fExtends[842] = 69;
	fExtends[843] = 69;
	fExtends[844] = 69;
	fExtends[845] = 69;
	fExtends[846] = 50;
	fExtends[847] = 50;
	fExtends[848] = 75;
	fExtends[849] = 75;
	fExtends[850] = 62;
	fExtends[851] = 62;
	fExtends[852] = 81;
	fExtends[853] = 81;
	fExtends[854] = 81;
	fExtends[855] = 81;
	fExtends[856] = 69;
	fExtends[857] = 69;
	fExtends[858] = 69;
	fExtends[859] = 69;
	fExtends[860] = 87;
	fExtends[861] = 87;
	fExtends[862] = 81;
	fExtends[863] = 81;
	fExtends[864] = 75;
	fExtends[865] = 75;
	fExtends[866] = 75;
	fExtends[867] = 75;
	fExtends[868] = 87;
	fExtends[869] = 87;
	fExtends[870] = 87;
	fExtends[871] = 87;
	fExtends[872] = 59;
	fExtends[873] = 59;
	fExtends[874] = 59;
	fExtends[875] = 59;
	fExtends[876] = 59;
	fExtends[877] = 59;
	fExtends[878] = 59;
	fExtends[879] = 59;
	fExtends[880] = 37;
	fExtends[881] = 37;
	fExtends[882] = 31;
	fExtends[883] = 31;
	fExtends[884] = 62;
	fExtends[885] = 62;
	fExtends[886] = 56;
	fExtends[887] = 56;
	fExtends[888] = 62;
	fExtends[889] = 63;
	fExtends[890] = 75;
	fExtends[891] = 75;
	fExtends[892] = 69;
	fExtends[893] = 69;
	fExtends[894] = 62;
	fExtends[895] = 62;
	fExtends[896] = 81;
	fExtends[897] = 81;
	fExtends[898] = 81;
	fExtends[899] = 81;
	fExtends[900] = 75;
	fExtends[901] = 75;
	fExtends[902] = 81;
	fExtends[903] = 81;
	fExtends[904] = 59;
	fExtends[905] = 59;
	fExtends[906] = 59;
	fExtends[907] = 59;
	fExtends[908] = 59;
	fExtends[909] = 59;
	fExtends[910] = 59;
	fExtends[911] = 59;
	fExtends[912] = 25;
	fExtends[913] = 25;
	fExtends[914] = 31;
	fExtends[915] = 31;
	fExtends[916] = 31;
	fExtends[917] = 31;
	fExtends[918] = 37;
	fExtends[919] = 37;
	fExtends[920] = 56;
	fExtends[921] = 56;
	fExtends[922] = 50;
	fExtends[923] = 50;
	fExtends[924] = 56;
	fExtends[925] = 56;
	fExtends[926] = 69;
	fExtends[927] = 69;
	fExtends[928] = 50;
	fExtends[929] = 50;
	fExtends[930] = 69;
	fExtends[931] = 69;
	fExtends[932] = 50;
	fExtends[933] = 50;
	fExtends[934] = 69;
	fExtends[935] = 69;
	fExtends[936] = 56;
	fExtends[937] = 56;
	fExtends[938] = 62;
	fExtends[939] = 62;
	fExtends[940] = 87;
	fExtends[941] = 88;
	fExtends[942] = 59;
	fExtends[943] = 59;
	fExtends[944] = 50;
	fExtends[945] = 50;
	fExtends[946] = 50;
	fExtends[947] = 50;
	fExtends[948] = 50;
	fExtends[949] = 50;
	fExtends[950] = 56;
	fExtends[951] = 56;
	fExtends[952] = 69;
	fExtends[953] = 69;
	fExtends[954] = 81;
	fExtends[955] = 81;
	fExtends[956] = 62;
	fExtends[957] = 62;
	fExtends[958] = 81;
	fExtends[959] = 81;
	fExtends[960] = 56;
	fExtends[961] = 56;
	fExtends[962] = 62;
	fExtends[963] = 62;
	fExtends[964] = 62;
	fExtends[965] = 62;
	fExtends[966] = 62;
	fExtends[967] = 62;
	fExtends[968] = 75;
	fExtends[969] = 75;
	fExtends[970] = 75;
	fExtends[971] = 75;
	fExtends[972] = 81;
	fExtends[973] = 81;
	fExtends[974] = 94;
	fExtends[975] = 94;
	fExtends[976] = 75;
	fExtends[977] = 75;
	fExtends[978] = 87;
	fExtends[979] = 87;
	fExtends[980] = 75;
	fExtends[981] = 75;
	fExtends[982] = 94;
	fExtends[983] = 94;
	fExtends[984] = 81;
	fExtends[985] = 81;
	fExtends[986] = 87;
	fExtends[987] = 87;
	fExtends[988] = 100;
	fExtends[989] = 100;
	fExtends[990] = 59;
	fExtends[991] = 59;
	fExtends[992] = 75;
	fExtends[993] = 75;
	fExtends[994] = 75;
	fExtends[995] = 75;
	fExtends[996] = 75;
	fExtends[997] = 75;
	fExtends[998] = 81;
	fExtends[999] = 81;
	fExtends[1000] = 81;
	fExtends[1001] = 81;
	fExtends[1002] = 94;
	fExtends[1003] = 94;
	fExtends[1004] = 87;
	fExtends[1005] = 87;
	fExtends[1006] = 87;
	fExtends[1007] = 87;
	fExtends[1008] = 19;
	fExtends[1009] = 19;
	fExtends[1010] = 37;
	fExtends[1011] = 37;
	fExtends[1012] = 59;
	fExtends[1013] = 59;
	fExtends[1014] = 59;
	fExtends[1015] = 59;
	fExtends[1016] = 59;
	fExtends[1017] = 59;
	fExtends[1018] = 59;
	fExtends[1019] = 59;
	fExtends[1020] = 59;
	fExtends[1021] = 59;
	fExtends[1022] = 59;
	fExtends[1023] = 59;
	fExtends[1024] = 59;
	fExtends[1025] = 59;
	fExtends[1026] = 59;
	fExtends[1027] = 59;
	fExtends[1028] = 59;
	fExtends[1029] = 59;
	fExtends[1030] = 59;
	fExtends[1031] = 59;
	fExtends[1032] = 59;
	fExtends[1033] = 59;
	fExtends[1034] = 59;
	fExtends[1035] = 59;
	fExtends[1036] = 59;
	fExtends[1037] = 59;
	fExtends[1038] = 59;
	fExtends[1039] = 59;
	fExtends[1040] = 59;
	fExtends[1041] = 59;
	fExtends[1042] = 59;
	fExtends[1043] = 59;
	fExtends[1044] = 59;
	fExtends[1045] = 59;
	fExtends[1046] = 59;
	fExtends[1047] = 59;
	fExtends[1048] = 59;
	fExtends[1049] = 59;
	fExtends[1050] = 59;
	fExtends[1051] = 59;
	fExtends[1052] = 59;
	fExtends[1053] = 59;
	fExtends[1054] = 59;
	fExtends[1055] = 59;
	fExtends[1056] = 32;
	fExtends[1057] = 32;
	fExtends[1058] = 33;
	fExtends[1059] = 45;
	fExtends[1060] = 45;
	fExtends[1061] = 50;
	fExtends[1062] = 66;
	fExtends[1063] = 66;
	fExtends[1064] = 63;
	fExtends[1065] = 79;
	fExtends[1066] = 79;
	fExtends[1067] = 43;
	fExtends[1068] = 43;
	fExtends[1069] = 43;
	fExtends[1070] = 45;
	fExtends[1071] = 45;
	fExtends[1072] = 45;
	fExtends[1073] = 83;
	fExtends[1074] = 83;
	fExtends[1075] = 83;
	fExtends[1076] = 83;
	fExtends[1077] = 83;
	fExtends[1078] = 59;
	fExtends[1079] = 59;
	fExtends[1080] = 59;
	fExtends[1081] = 59;
	fExtends[1082] = 59;
	fExtends[1083] = 59;
	fExtends[1084] = 59;
	fExtends[1085] = 59;
	fExtends[1086] = 59;
	fExtends[1087] = 59;
	fExtends[1088] = 57;
	fExtends[1089] = 45;
	fExtends[1090] = 57;
	fExtends[1091] = 46;
	fExtends[1092] = 46;
	fExtends[1093] = 50;
	fExtends[1094] = 68;
	fExtends[1095] = 69;
	fExtends[1096] = 59;
	fExtends[1097] = 59;
	fExtends[1098] = 59;
	fExtends[1099] = 59;
	fExtends[1100] = 59;
	fExtends[1101] = 59;
	fExtends[1102] = 59;
	fExtends[1103] = 59;
	fExtends[1104] = 77;
	fExtends[1105] = 91;
	fExtends[1106] = 87;
	fExtends[1107] = 89;
	fExtends[1108] = 50;
	fExtends[1109] = 64;
	fExtends[1110] = 60;
	fExtends[1111] = 64;
	fExtends[1112] = 59;
	fExtends[1113] = 59;
	fExtends[1114] = 59;
	fExtends[1115] = 59;
	fExtends[1116] = 59;
	fExtends[1117] = 59;
	fExtends[1118] = 59;
	fExtends[1119] = 59;
	fExtends[1120] = 60;
	fExtends[1121] = 84;
	fExtends[1122] = 59;
	fExtends[1123] = 59;
	fExtends[1124] = 59;
	fExtends[1125] = 59;
	fExtends[1126] = 59;
	fExtends[1127] = 59;
	fExtends[1128] = 59;
	fExtends[1129] = 59;
	fExtends[1130] = 59;
	fExtends[1131] = 59;
	fExtends[1132] = 59;
	fExtends[1133] = 59;
	fExtends[1134] = 59;
	fExtends[1135] = 59;
	fExtends[1136] = 49;
	fExtends[1137] = 61;
	fExtends[1138] = 33;
	fExtends[1139] = 50;
	fExtends[1140] = 74;
	fExtends[1141] = 110;
	fExtends[1142] = 80;
	fExtends[1143] = 95;
	fExtends[1144] = 45;
	fExtends[1145] = 60;
	fExtends[1146] = 60;
	fExtends[1147] = 56;
	fExtends[1148] = 56;
	fExtends[1149] = 69;
	fExtends[1150] = 79;
	fExtends[1151] = 84;
	fExtends[1152] = 45;
	fExtends[1153] = 50;
	fExtends[1154] = 34;
	fExtends[1155] = 83;
	fExtends[1156] = 46;
	fExtends[1157] = 83;
	fExtends[1158] = 45;
	fExtends[1159] = 69;
	fExtends[1160] = 47;
	fExtends[1161] = 82;
	fExtends[1162] = 38;
	fExtends[1163] = 38;
	fExtends[1164] = 39;
	fExtends[1165] = 40;
	fExtends[1166] = 40;
	fExtends[1167] = 44;
	fExtends[1168] = 59;
	fExtends[1169] = 59;
	fExtends[1170] = 59;
	fExtends[1171] = 59;
	fExtends[1172] = 59;
	fExtends[1173] = 59;
	fExtends[1174] = 59;
	fExtends[1175] = 59;
	fExtends[1176] = 59;
	fExtends[1177] = 59;
	fExtends[1178] = 59;
	fExtends[1179] = 59;
	fExtends[1180] = 59;
	fExtends[1181] = 59;
	fExtends[1182] = 59;
	fExtends[1183] = 59;
	fExtends[1184] = 68;
	fExtends[1185] = 68;
	fExtends[1186] = 17;
	fExtends[1187] = 17;
	fExtends[1188] = 68;
	fExtends[1189] = 68;
	fExtends[1190] = 20;
	fExtends[1191] = 20;
	fExtends[1192] = 18;
	fExtends[1193] = 18;
	fExtends[1194] = 10;
	fExtends[1195] = 10;
	fExtends[1196] = 47;
	fExtends[1197] = 47;
	fExtends[1198] = 47;
	fExtends[1199] = 47;
	fExtends[1200] = 68;
	fExtends[1201] = 68;
	fExtends[1202] = 68;
	fExtends[1203] = 68;
	fExtends[1204] = 68;
	fExtends[1205] = 68;
	fExtends[1206] = 47;
	fExtends[1207] = 48;
	fExtends[1208] = 76;
	fExtends[1209] = 77;
	fExtends[1210] = 73;
	fExtends[1211] = 73;
	fExtends[1212] = 68;
	fExtends[1213] = 68;
	fExtends[1214] = 59;
	fExtends[1215] = 59;
	fExtends[1216] = 121;
	fExtends[1217] = 121;
	fExtends[1218] = 147;
	fExtends[1219] = 147;
	fExtends[1220] = 121;
	fExtends[1221] = 121;
	fExtends[1222] = 121;
	fExtends[1223] = 121;
	fExtends[1224] = 143;
	fExtends[1225] = 143;
	fExtends[1226] = 147;
	fExtends[1227] = 147;
	fExtends[1228] = 87;
	fExtends[1229] = 87;
	fExtends[1230] = 31;
	fExtends[1231] = 100;
	fExtends[1232] = 50;
	fExtends[1233] = 77;
	fExtends[1234] = 133;
	fExtends[1235] = 37;
	fExtends[1236] = 75;
	fExtends[1237] = 105;
	fExtends[1238] = 140;
	fExtends[1239] = 59;
	fExtends[1240] = 59;
	fExtends[1241] = 59;
	fExtends[1242] = 59;
	fExtends[1243] = 59;
	fExtends[1244] = 59;
	fExtends[1245] = 59;
	fExtends[1246] = 59;
	fExtends[1247] = 59;
	fExtends[1248] = 76;
	fExtends[1249] = 25;
	fExtends[1250] = 25;
	fExtends[1251] = 57;
	fExtends[1252] = 57;
	fExtends[1253] = 54;
	fExtends[1254] = 50;
	fExtends[1255] = 64;
	fExtends[1256] = 73;
	fExtends[1257] = 85;
	fExtends[1258] = 97;
	fExtends[1259] = 108;
	fExtends[1260] = 121;
	fExtends[1261] = 134;
	fExtends[1262] = 157;
	fExtends[1263] = 75;
	fExtends[1264] = 66;
	fExtends[1265] = 75;
	fExtends[1266] = 50;
	fExtends[1267] = 59;
	fExtends[1268] = 56;
	fExtends[1269] = 56;
	fExtends[1270] = 50;
	fExtends[1271] = 59;
	fExtends[1272] = 59;
	fExtends[1273] = 59;
	fExtends[1274] = 59;
	fExtends[1275] = 59;
	fExtends[1276] = 59;
	fExtends[1277] = 59;
	fExtends[1278] = 59;
	fExtends[1279] = 59;
	fExtends[1280] = 107;
	fExtends[1281] = 152;
	fExtends[1282] = 247;
	fExtends[1283] = 25;
	fExtends[1284] = 106;
	fExtends[1285] = 25;
	fExtends[1286] = 59;
	fExtends[1287] = 59;
	fExtends[1288] = 59;
	fExtends[1289] = 59;
	fExtends[1290] = 59;
	fExtends[1291] = 59;
	fExtends[1292] = 59;
	fExtends[1293] = 59;
	fExtends[1294] = 59;
	fExtends[1295] = 59;
	fExtends[1296] = 77;
	fExtends[1297] = 177;
	fExtends[1298] = 168;
	fExtends[1299] = 175;
	fExtends[1300] = 133;
	fExtends[1301] = 263;
	fExtends[1302] = 257;
	fExtends[1303] = 150;
	fExtends[1304] = 286;
	fExtends[1305] = 276;
	fExtends[1306] = 27;
	fExtends[1307] = 27;
	fExtends[1308] = 159;
	fExtends[1309] = 246;
	fExtends[1310] = 267;
	fExtends[1311] = 213;
	fExtends[1312] = 73;
	fExtends[1313] = 87;
	fExtends[1314] = 73;
	fExtends[1315] = 55;
	fExtends[1316] = 46;
	fExtends[1317] = 49;
	fExtends[1318] = 62;
	fExtends[1319] = 425;
	fExtends[1320] = 355;
	fExtends[1321] = 283;
	fExtends[1322] = 214;
	fExtends[1323] = 145;
	fExtends[1324] = 165;
	fExtends[1325] = 159;
	fExtends[1326] = 154;
	fExtends[1327] = 122;
	fExtends[1328] = 166;
	fExtends[1329] = 214;
	fExtends[1330] = 262;
	fExtends[1331] = 310;
	fExtends[1332] = 124;
	fExtends[1333] = 99;
	fExtends[1334] = 121;
	fExtends[1335] = 169;
	fExtends[1336] = 240;
	fExtends[1337] = 146;
	fExtends[1338] = 215;
	fExtends[1339] = 193;
	fExtends[1340] = 125;
	fExtends[1341] = 149;
	fExtends[1342] = 150;
	fExtends[1343] = 150;
	fExtends[1344] = 311;
	fExtends[1345] = 27;
	fExtends[1346] = 22;
	fExtends[1347] = 22;
	fExtends[1348] = 15;
	fExtends[1349] = 15;
	fExtends[1350] = 34;
	fExtends[1351] = 56;
	fExtends[1352] = 35;
	fExtends[1353] = 45;
	fExtends[1354] = 59;
	fExtends[1355] = 59;
	fExtends[1356] = 59;
	fExtends[1357] = 59;
	fExtends[1358] = 59;
	fExtends[1359] = 59;
	fExtends[1360] = 73;
	fExtends[1361] = 93;
	fExtends[1362] = 133;
	fExtends[1363] = 67;
	fExtends[1364] = 67;
	fExtends[1365] = 81;
	fExtends[1366] = 59;
	fExtends[1367] = 59;
	fExtends[1368] = 59;
	fExtends[1369] = 59;
	fExtends[1370] = 59;
	fExtends[1371] = 59;
	fExtends[1372] = 59;
	fExtends[1373] = 59;
	fExtends[1374] = 59;
	fExtends[1375] = 59;
	fExtends[1376] = 71;
	fExtends[1377] = 39;
	fExtends[1378] = 65;
	fExtends[1379] = 39;
	fExtends[1380] = 101;
	fExtends[1381] = 101;
	fExtends[1382] = 104;
	fExtends[1383] = 92;
	fExtends[1384] = 91;
	fExtends[1385] = 92;
	fExtends[1386] = 44;
	fExtends[1387] = 43;
	fExtends[1388] = 146;
	fExtends[1389] = 146;
	fExtends[1390] = 197;
	fExtends[1391] = 100;
	fExtends[1392] = 89;
	fExtends[1393] = 32;
	fExtends[1394] = 32;
	fExtends[1395] = 44;
	fExtends[1396] = 44;
	fExtends[1397] = 75;
	fExtends[1398] = 100;
	fExtends[1399] = 78;
	fExtends[1400] = 78;
	fExtends[1401] = 40;
	fExtends[1402] = 40;
	fExtends[1403] = 63;
	fExtends[1404] = 63;
	fExtends[1405] = 100;
	fExtends[1406] = 104;
	fExtends[1407] = 97;
	fExtends[1408] = 97;
	fExtends[1409] = 28;
	fExtends[1410] = 57;
	fExtends[1411] = 5;
	fExtends[1412] = 44;
	fExtends[1413] = 218;
	fExtends[1414] = 218;
	fExtends[1415] = 229;
	fExtends[1416] = 117;
	fExtends[1417] = 122;
	fExtends[1418] = 59;
	fExtends[1419] = 59;
	fExtends[1420] = 59;
	fExtends[1421] = 59;
	fExtends[1422] = 59;
	fExtends[1423] = 59;
	fExtends[1424] = 70;
	fExtends[1425] = 57;
	fExtends[1426] = 86;
	fExtends[1427] = 56;
	fExtends[1428] = 37;
	fExtends[1429] = 58;
	fExtends[1430] = 46;
	fExtends[1431] = 139;
	fExtends[1432] = 77;
	fExtends[1433] = 67;
	fExtends[1434] = 23;
	fExtends[1435] = 96;
	fExtends[1436] = 81;
	fExtends[1437] = 52;
	fExtends[1438] = 71;
	fExtends[1439] = 0;
	fExtends[1440] = 88;
	fExtends[1441] = 23;
	fExtends[1442] = 107;
	fExtends[1443] = 80;
	fExtends[1444] = 33;
	fExtends[1445] = 65;
	fExtends[1446] = 96;
	fExtends[1447] = 55;
	fExtends[1448] = 110;
	fExtends[1449] = 59;
	fExtends[1450] = 59;
	fExtends[1451] = 59;
	fExtends[1452] = 59;
	fExtends[1453] = 59;
	fExtends[1454] = 59;
	fExtends[1455] = 59;
	fExtends[1456] = 222;
	fExtends[1457] = 226;
	fExtends[1458] = 224;
	fExtends[1459] = 175;
	fExtends[1460] = 299;
	fExtends[1461] = 230;
	fExtends[1462] = 222;
	fExtends[1463] = 295;
	fExtends[1464] = 210;
	fExtends[1465] = 282;
	fExtends[1466] = 216;
	fExtends[1467] = 190;
	fExtends[1468] = 261;
	fExtends[1469] = 198;
	fExtends[1470] = 221;
	fExtends[1471] = 221;
	fExtends[1472] = 274;
	fExtends[1473] = 203;
	fExtends[1474] = 203;
	fExtends[1475] = 256;
	fExtends[1476] = 255;
	fExtends[1477] = 139;
	fExtends[1478] = 245;
	fExtends[1479] = 244;
	fExtends[1480] = 172;
	fExtends[1481] = 59;
	fExtends[1482] = 59;
	fExtends[1483] = 59;
	fExtends[1484] = 59;
	fExtends[1485] = 59;
	fExtends[1486] = 59;
	fExtends[1487] = 59;
	fExtends[1488] = 64;
	fExtends[1489] = 85;
	fExtends[1490] = 162;
	fExtends[1491] = 239;
	fExtends[1492] = 64;
	fExtends[1493] = 98;
	fExtends[1494] = 230;
	fExtends[1495] = 59;
	fExtends[1496] = 98;
	fExtends[1497] = 230;
	fExtends[1498] = 82;
	fExtends[1499] = 182;
	fExtends[1500] = 274;
	fExtends[1501] = 85;
	fExtends[1502] = 162;
	fExtends[1503] = 239;
	fExtends[1504] = 59;
	fExtends[1505] = 76;
	fExtends[1506] = 61;
	fExtends[1507] = 79;
	fExtends[1508] = 124;
	fExtends[1509] = 54;
	fExtends[1510] = 61;
	fExtends[1511] = 61;
	fExtends[1512] = 58;
	fExtends[1513] = 61;
	fExtends[1514] = 61;
	fExtends[1515] = 60;
	fExtends[1516] = 82;
	fExtends[1517] = 182;
	fExtends[1518] = 274;
	fExtends[1519] = 61;
	fExtends[1520] = 89;
	fExtends[1521] = 89;
	fExtends[1522] = 115;
	fExtends[1523] = 115;
	fExtends[1524] = 61;
	fExtends[1525] = 61;
	fExtends[1526] = 61;
	fExtends[1527] = 61;
	fExtends[1528] = 61;
	fExtends[1529] = 61;
	fExtends[1530] = 141;
	fExtends[1531] = 75;
	fExtends[1532] = 75;
	fExtends[1533] = 95;
	fExtends[1534] = 95;
	fExtends[1535] = 88;
	fExtends[1536] = 96;
	fExtends[1537] = 94;
	fExtends[1538] = 118;
	fExtends[1539] = 111;
	fExtends[1540] = 40;
	fExtends[1541] = 40;
	fExtends[1542] = 40;
	fExtends[1543] = 56;
	fExtends[1544] = 60;
	fExtends[1545] = 60;
	fExtends[1546] = 77;
	fExtends[1547] = 61;
	fExtends[1548] = 59;
	fExtends[1549] = 59;
	fExtends[1550] = 59;
	fExtends[1551] = 59;
	fExtends[1552] = 62;
	fExtends[1553] = 62;
	fExtends[1554] = 50;
	fExtends[1555] = 50;
	fExtends[1556] = 40;
	fExtends[1557] = 40;
	fExtends[1558] = 78;
	fExtends[1559] = 78;
	fExtends[1560] = 43;
	fExtends[1561] = 39;
	fExtends[1562] = 40;
	fExtends[1563] = 63;
	fExtends[1564] = 50;
	fExtends[1565] = 63;
	fExtends[1566] = 54;
	fExtends[1567] = 63;
	fExtends[1568] = 143;
	fExtends[1569] = 143;
	fExtends[1570] = 32;
	fExtends[1571] = 53;
	fExtends[1572] = 34;
	fExtends[1573] = 34;
	fExtends[1574] = 189;
	fExtends[1575] = 43;
	fExtends[1576] = 43;
	fExtends[1577] = 43;
	fExtends[1578] = 43;
	fExtends[1579] = 59;
	fExtends[1580] = 59;
	fExtends[1581] = 59;
	fExtends[1582] = 59;
	fExtends[1583] = 59;
	fExtends[1584] = 40;
	fExtends[1585] = 40;
	fExtends[1586] = 60;
	fExtends[1587] = 54;
	fExtends[1588] = 46;
	fExtends[1589] = 46;
	fExtends[1590] = 80;
	fExtends[1591] = 50;
	fExtends[1592] = 65;
	fExtends[1593] = 69;
	fExtends[1594] = 89;
	fExtends[1595] = 43;
	fExtends[1596] = 59;
	fExtends[1597] = 59;
	fExtends[1598] = 59;
	fExtends[1599] = 59;
	fExtends[1600] = 75;
	fExtends[1601] = 75;
	fExtends[1602] = 75;
	fExtends[1603] = 75;
	fExtends[1604] = 50;
	fExtends[1605] = 32;
	fExtends[1606] = 26;
	fExtends[1607] = 69;
	fExtends[1608] = 136;
	fExtends[1609] = 134;
	fExtends[1610] = 133;
	fExtends[1611] = 15;
	fExtends[1612] = 59;
	fExtends[1613] = 59;
	fExtends[1614] = 59;
	fExtends[1615] = 59;
	fExtends[1616] = 204;
	fExtends[1617] = 99;
	fExtends[1618] = 42;
	fExtends[1619] = 49;
	fExtends[1620] = 17;
	fExtends[1621] = 90;
	fExtends[1622] = 150;
	fExtends[1623] = 58;
	fExtends[1624] = 59;
	fExtends[1625] = 221;
	fExtends[1626] = 84;
	fExtends[1627] = 90;
	fExtends[1628] = 90;
	fExtends[1629] = 83;
	fExtends[1630] = 84;
	fExtends[1631] = 85;
	fExtends[1632] = 84;
	fExtends[1633] = 85;
	fExtends[1634] = 85;
	fExtends[1635] = 100;
	fExtends[1636] = 88;
	fExtends[1637] = 88;
	fExtends[1638] = 85;
	fExtends[1639] = 99;
	fExtends[1640] = 89;
	fExtends[1641] = 89;
	fExtends[1642] = 115;
	fExtends[1643] = 115;
	fExtends[1644] = 143;
	fExtends[1645] = 143;
	fExtends[1646] = 78;
	fExtends[1647] = 78;
	fExtends[1648] = 78;
	fExtends[1649] = 78;
	fExtends[1650] = 41;
	fExtends[1651] = 41;
	fExtends[1652] = 178;
	fExtends[1653] = 178;
	fExtends[1654] = 33;
	fExtends[1655] = 33;
	fExtends[1656] = 59;
	fExtends[1657] = 59;
	fExtends[1658] = 59;
	fExtends[1659] = 59;
	fExtends[1660] = 59;
	fExtends[1661] = 59;
	fExtends[1662] = 59;
	fExtends[1663] = 59;
	fExtends[1664] = 50;
	fExtends[1665] = 50;
	fExtends[1666] = 50;
	fExtends[1667] = 50;
	fExtends[1668] = 114;
	fExtends[1669] = 276;
	fExtends[1670] = 160;
	fExtends[1671] = 107;
	fExtends[1672] = 82;
	fExtends[1673] = 113;
	fExtends[1674] = 172;
	fExtends[1675] = 113;
	fExtends[1676] = 107;
	fExtends[1677] = 45;
	fExtends[1678] = 69;
	fExtends[1679] = 140;
	fExtends[1680] = 150;
	fExtends[1681] = 193;
	fExtends[1682] = 145;
	fExtends[1683] = 164;
	fExtends[1684] = 36;
	fExtends[1685] = 96;
	fExtends[1686] = 96;
	fExtends[1687] = 57;
	fExtends[1688] = 127;
	fExtends[1689] = 57;
	fExtends[1690] = 57;
	fExtends[1691] = 82;
	fExtends[1692] = 108;
	fExtends[1693] = 100;
	fExtends[1694] = 59;
	fExtends[1695] = 59;
	fExtends[1696] = 240;
	fExtends[1697] = 240;
	fExtends[1698] = 239;
	fExtends[1699] = 239;
	fExtends[1700] = 239;
	fExtends[1701] = 239;
	fExtends[1702] = 240;
	fExtends[1703] = 239;
	fExtends[1704] = 239;
	fExtends[1705] = 240;
	fExtends[1706] = 251;
	fExtends[1707] = 293;
	fExtends[1708] = 314;
	fExtends[1709] = 293;
	fExtends[1710] = 235;
	fExtends[1711] = 211;
	fExtends[1712] = 239;
	fExtends[1713] = 227;
	fExtends[1714] = 239;
	fExtends[1715] = 59;
	fExtends[1716] = 59;
	fExtends[1717] = 59;
	fExtends[1718] = 59;
	fExtends[1719] = 59;
	fExtends[1720] = 59;
	fExtends[1721] = 59;
	fExtends[1722] = 59;
	fExtends[1723] = 59;
	fExtends[1724] = 59;
	fExtends[1725] = 59;
	fExtends[1726] = 59;
	fExtends[1727] = 59;
	fExtends[1728] = 187;
	fExtends[1729] = 174;
	fExtends[1730] = 210;
	fExtends[1731] = 204;
	fExtends[1732] = 204;
	fExtends[1733] = 204;
	fExtends[1734] = 204;
	fExtends[1735] = 204;
	fExtends[1736] = 204;
	fExtends[1737] = 59;
	fExtends[1738] = 59;
	fExtends[1739] = 59;
	fExtends[1740] = 59;
	fExtends[1741] = 59;
	fExtends[1742] = 59;
	fExtends[1743] = 59;
	fExtends[1744] = 189;
	fExtends[1745] = 197;
	fExtends[1746] = 197;
	fExtends[1747] = 83;
	fExtends[1748] = 77;
	fExtends[1749] = 155;
	fExtends[1750] = 83;
	fExtends[1751] = 108;
	fExtends[1752] = 255;
	fExtends[1753] = 255;
	fExtends[1754] = 255;
	fExtends[1755] = 182;
	fExtends[1756] = 223;
	fExtends[1757] = 246;
	fExtends[1758] = 118;
	fExtends[1759] = 234;
	fExtends[1760] = 208;
	fExtends[1761] = 157;
	fExtends[1762] = 146;
	fExtends[1763] = 153;
	fExtends[1764] = 135;
	fExtends[1765] = 59;
	fExtends[1766] = 59;
	fExtends[1767] = 59;
	fExtends[1768] = 59;
	fExtends[1769] = 59;
	fExtends[1770] = 59;
	fExtends[1771] = 59;
	fExtends[1772] = 59;
	fExtends[1773] = 59;
	fExtends[1774] = 59;
	fExtends[1775] = 59;
	fExtends[1776] = 157;
	fExtends[1777] = 186;
	fExtends[1778] = 124;
	fExtends[1779] = 200;
	fExtends[1780] = 140;
	fExtends[1781] = 240;
	fExtends[1782] = 140;
	fExtends[1783] = 182;
	fExtends[1784] = 107;
	fExtends[1785] = 59;
	fExtends[1786] = 216;
	fExtends[1787] = 179;
	fExtends[1788] = 200;
	fExtends[1789] = 59;
	fExtends[1790] = 59;
	fExtends[1791] = 59;
	fExtends[1792] = 207;
	fExtends[1793] = 195;
	fExtends[1794] = 59;
	fExtends[1795] = 59;
	fExtends[1796] = 59;
	fExtends[1797] = 59;
	fExtends[1798] = 59;
	fExtends[1799] = 59;
	fExtends[1800] = 59;
	fExtends[1801] = 59;
	fExtends[1802] = 59;
	fExtends[1803] = 59;
	fExtends[1804] = 59;
	fExtends[1805] = 59;
	fExtends[1806] = 59;
	fExtends[1807] = 59;
	fExtends[1808] = 268;
	fExtends[1809] = 90;
	fExtends[1810] = 92;
	fExtends[1811] = 108;
	fExtends[1812] = 136;
	fExtends[1813] = 136;
	fExtends[1814] = 50;
	fExtends[1815] = 88;
	fExtends[1816] = 100;
	fExtends[1817] = 178;
	fExtends[1818] = 108;
	fExtends[1819] = 59;
	fExtends[1820] = 59;
	fExtends[1821] = 59;
	fExtends[1822] = 59;
	fExtends[1823] = 59;
	fExtends[1824] = 43;
	fExtends[1825] = 157;
	fExtends[1826] = 157;
	fExtends[1827] = 157;
	fExtends[1828] = 156;
	fExtends[1829] = 194;
	fExtends[1830] = 252;
	fExtends[1831] = 50;
	fExtends[1832] = 159;
	fExtends[1833] = 241;
	fExtends[1834] = 174;
	fExtends[1835] = 59;
	fExtends[1836] = 59;
	fExtends[1837] = 59;
	fExtends[1838] = 59;
	fExtends[1839] = 59;
	fExtends[1840] = 204;
	fExtends[1841] = 197;
	fExtends[1842] = 137;
	fExtends[1843] = 137;
	fExtends[1844] = 170;
	fExtends[1845] = 59;
	fExtends[1846] = 59;
	fExtends[1847] = 59;
	fExtends[1848] = 59;
	fExtends[1849] = 59;
	fExtends[1850] = 59;
	fExtends[1851] = 59;
	fExtends[1852] = 59;
	fExtends[1853] = 59;
	fExtends[1854] = 59;
	fExtends[1855] = 59;
	fExtends[1856] = 72;
	fExtends[1857] = 81;
	fExtends[1858] = 206;
	fExtends[1859] = 81;
	fExtends[1860] = 205;
	fExtends[1861] = 147;
	fExtends[1862] = 146;
	fExtends[1863] = 52;
	fExtends[1864] = 170;
	fExtends[1865] = 59;
	fExtends[1866] = 59;
	fExtends[1867] = 59;
	fExtends[1868] = 59;
	fExtends[1869] = 59;
	fExtends[1870] = 59;
	fExtends[1871] = 59;
	fExtends[1872] = 195;
	fExtends[1873] = 142;
	fExtends[1874] = 198;
	fExtends[1875] = 175;
	fExtends[1876] = 279;
	fExtends[1877] = 194;
	fExtends[1878] = 161;
	fExtends[1879] = 223;
	fExtends[1880] = 244;
	fExtends[1881] = 205;
	fExtends[1882] = 176;
	fExtends[1883] = 59;
	fExtends[1884] = 59;
	fExtends[1885] = 59;
	fExtends[1886] = 59;
	fExtends[1887] = 59;
	fExtends[1888] = 254;
	fExtends[1889] = 169;
	fExtends[1890] = 187;
	fExtends[1891] = 143;
	fExtends[1892] = 133;
	fExtends[1893] = 182;
	fExtends[1894] = 172;
	fExtends[1895] = 100;
	fExtends[1896] = 59;
	fExtends[1897] = 59;
	fExtends[1898] = 59;
	fExtends[1899] = 59;
	fExtends[1900] = 143;
	fExtends[1901] = 59;
	fExtends[1902] = 59;
	fExtends[1903] = 59;
	fExtends[1904] = 46;
	fExtends[1905] = 46;
	fExtends[1906] = 87;
	fExtends[1907] = 87;
	fExtends[1908] = 47;
	fExtends[1909] = 47;
	fExtends[1910] = 88;
	fExtends[1911] = 87;
	fExtends[1912] = 46;
	fExtends[1913] = 46;
	fExtends[1914] = 87;
	fExtends[1915] = 87;
	fExtends[1916] = 46;
	fExtends[1917] = 46;
	fExtends[1918] = 88;
	fExtends[1919] = 87;
	fExtends[1920] = 28;
	fExtends[1921] = 28;
	fExtends[1922] = 83;
	fExtends[1923] = 83;
	fExtends[1924] = 22;
	fExtends[1925] = 22;
	fExtends[1926] = 80;
	fExtends[1927] = 80;
	fExtends[1928] = 44;
	fExtends[1929] = 44;
	fExtends[1930] = 95;
	fExtends[1931] = 95;
	fExtends[1932] = 44;
	fExtends[1933] = 44;
	fExtends[1934] = 95;
	fExtends[1935] = 95;
	fExtends[1936] = 44;
	fExtends[1937] = 44;
	fExtends[1938] = 95;
	fExtends[1939] = 95;
	fExtends[1940] = 44;
	fExtends[1941] = 44;
	fExtends[1942] = 96;
	fExtends[1943] = 95;
	fExtends[1944] = 64;
	fExtends[1945] = 64;
	fExtends[1946] = 64;
	fExtends[1947] = 64;
	fExtends[1948] = 64;
	fExtends[1949] = 64;
	fExtends[1950] = 64;
	fExtends[1951] = 64;
	fExtends[1952] = 64;
	fExtends[1953] = 64;
	fExtends[1954] = 55;
	fExtends[1955] = 55;
	fExtends[1956] = 91;
	fExtends[1957] = 91;
	fExtends[1958] = 55;
	fExtends[1959] = 55;
	fExtends[1960] = 92;
	fExtends[1961] = 91;
	fExtends[1962] = 55;
	fExtends[1963] = 55;
	fExtends[1964] = 92;
	fExtends[1965] = 91;
	fExtends[1966] = 46;
	fExtends[1967] = 46;
	fExtends[1968] = 76;
	fExtends[1969] = 76;
	fExtends[1970] = 46;
	fExtends[1971] = 46;
	fExtends[1972] = 46;
	fExtends[1973] = 88;
	fExtends[1974] = 87;
	fExtends[1975] = 46;
	fExtends[1976] = 46;
	fExtends[1977] = 88;
	fExtends[1978] = 87;
	fExtends[1979] = 50;
	fExtends[1980] = 50;
	fExtends[1981] = 100;
	fExtends[1982] = 99;
	fExtends[1983] = 50;
	fExtends[1984] = 50;
	fExtends[1985] = 99;
	fExtends[1986] = 99;
	fExtends[1987] = 50;
	fExtends[1988] = 50;
	fExtends[1989] = 99;
	fExtends[1990] = 99;
	fExtends[1991] = 46;
	fExtends[1992] = 46;
	fExtends[1993] = 87;
	fExtends[1994] = 87;
	fExtends[1995] = 112;
	fExtends[1996] = 112;
	fExtends[1997] = 112;
	fExtends[1998] = 112;
	fExtends[1999] = 112;
	fExtends[2000] = 112;
	fExtends[2001] = 28;
	fExtends[2002] = 28;
	fExtends[2003] = 29;
	fExtends[2004] = 29;
	fExtends[2005] = 81;
	fExtends[2006] = 81;
	fExtends[2007] = 82;
	fExtends[2008] = 82;
	fExtends[2009] = 59;
	fExtends[2010] = 59;
	fExtends[2011] = 109;
	fExtends[2012] = 89;
	fExtends[2013] = 80;
	fExtends[2014] = 19;
	fExtends[2015] = 81;
	fExtends[2016] = 80;
	fExtends[2017] = 111;
	fExtends[2018] = 57;
	fExtends[2019] = 104;
	fExtends[2020] = 104;
	fExtends[2021] = 102;
	fExtends[2022] = 61;
	fExtends[2023] = 274;
	fExtends[2024] = 11;
	fExtends[2025] = 94;
	fExtends[2026] = 80;
	fExtends[2027] = 79;
	fExtends[2028] = 81;
	fExtends[2029] = 105;
	fExtends[2030] = 105;
	fExtends[2031] = 76;
	fExtends[2032] = 89;
	fExtends[2033] = 156;
	fExtends[2034] = 156;
	fExtends[2035] = 156;
	fExtends[2036] = 101;
	fExtends[2037] = 57;
	fExtends[2038] = 57;
	fExtends[2039] = 57;
	fExtends[2040] = 57;
	fExtends[2041] = 57;
	fExtends[2042] = 141;
	fExtends[2043] = 141;
	fExtends[2044] = 142;
	fExtends[2045] = 25;
	fExtends[2046] = 107;
	fExtends[2047] = 107;
	fExtends[2048] = 107;
	fExtends[2049] = 126;
	fExtends[2050] = 107;
	fExtends[2051] = 107;
	fExtends[2052] = 107;
	fExtends[2053] = 31;
	fExtends[2054] = 56;
	fExtends[2055] = 25;
	fExtends[2056] = 49;
	fExtends[2057] = 87;
	fExtends[2058] = 87;
	fExtends[2059] = 85;
	fExtends[2060] = 51;
	fExtends[2061] = 32;
	fExtends[2062] = 156;
	fExtends[2063] = 59;
	fExtends[2064] = 52;
	fExtends[2065] = 83;
	fExtends[2066] = 52;
	fExtends[2067] = 96;
	fExtends[2068] = 56;
	fExtends[2069] = 57;
	fExtends[2070] = 87;
	fExtends[2071] = 46;
	fExtends[2072] = 26;
	fExtends[2073] = 26;
	fExtends[2074] = 61;
	fExtends[2075] = 66;
	fExtends[2076] = 26;
	fExtends[2077] = 156;
	fExtends[2078] = 128;
	fExtends[2079] = 107;
	fExtends[2080] = 146;
	fExtends[2081] = 156;
	fExtends[2082] = 200;
	fExtends[2083] = 200;
	fExtends[2084] = 200;
	fExtends[2085] = 200;
	fExtends[2086] = 200;
	fExtends[2087] = 200;
	fExtends[2088] = 200;
	fExtends[2089] = 200;
	fExtends[2090] = 200;
	fExtends[2091] = 200;
	fExtends[2092] = 200;
	fExtends[2093] = 200;
	fExtends[2094] = 200;
	fExtends[2095] = 200;
	fExtends[2096] = 102;
	fExtends[2097] = 107;
	fExtends[2098] = 229;
	fExtends[2099] = 108;
	fExtends[2100] = 108;
	fExtends[2101] = 108;
	fExtends[2102] = 108;
	fExtends[2103] = 109;
	fExtends[2104] = 108;
	fExtends[2105] = 108;
	fExtends[2106] = 108;
	fExtends[2107] = 108;
	fExtends[2108] = 108;
	fExtends[2109] = 57;
	fExtends[2110] = 57;
	fExtends[2111] = 57;
	fExtends[2112] = 79;
	fExtends[2113] = 39;
	fExtends[2114] = 54;
	fExtends[2115] = 72;
	fExtends[2116] = 72;
	fExtends[2117] = 145;
	fExtends[2118] = 46;
	fExtends[2119] = 46;
	fExtends[2120] = 79;
	fExtends[2121] = 167;
	fExtends[2122] = 200;
	fExtends[2123] = 200;
	fExtends[2124] = 200;
	fExtends[2125] = 200;
	fExtends[2126] = 200;
	fExtends[2127] = 200;
	fExtends[2128] = 72;
	fExtends[2129] = 73;
	fExtends[2130] = 106;
	fExtends[2131] = 106;
	fExtends[2132] = 140;
	fExtends[2133] = 140;
	fExtends[2134] = 173;
	fExtends[2135] = 172;
	fExtends[2136] = 28;
	fExtends[2137] = 30;
	fExtends[2138] = 28;
	fExtends[2139] = 200;
	fExtends[2140] = 200;
	fExtends[2141] = 200;
	fExtends[2142] = 200;
	fExtends[2143] = 200;
	fExtends[2144] = 141;
	fExtends[2145] = 142;
	fExtends[2146] = 59;
	fExtends[2147] = 59;
	fExtends[2148] = 149;
	fExtends[2149] = 149;
	fExtends[2150] = 149;
	fExtends[2151] = 149;
	fExtends[2152] = 85;
	fExtends[2153] = 85;
	fExtends[2154] = 250;
	fExtends[2155] = 253;
	fExtends[2156] = 59;
	fExtends[2157] = 59;
	fExtends[2158] = 59;
	fExtends[2159] = 59;
	fExtends[2160] = 95;
	fExtends[2161] = 95;
	fExtends[2162] = 79;
	fExtends[2163] = 112;
	fExtends[2164] = 65;
	fExtends[2165] = 33;
	fExtends[2166] = 33;
	fExtends[2167] = 31;
	fExtends[2168] = 31;
	fExtends[2169] = 21;
	fExtends[2170] = 21;
	fExtends[2171] = 63;
	fExtends[2172] = 200;
	fExtends[2173] = 59;
	fExtends[2174] = 59;
	fExtends[2175] = 59;
	fExtends[2176] = 64;
	fExtends[2177] = 49;
	fExtends[2178] = 64;
	fExtends[2179] = 59;
	fExtends[2180] = 61;
	fExtends[2181] = 63;
	fExtends[2182] = 61;
	fExtends[2183] = 61;
	fExtends[2184] = 63;
	fExtends[2185] = 61;
	fExtends[2186] = 22;
	fExtends[2187] = 59;
	fExtends[2188] = 59;
	fExtends[2189] = 59;
	fExtends[2190] = 59;
	fExtends[2191] = 59;
	fExtends[2192] = 80;
	fExtends[2193] = 40;
	fExtends[2194] = 40;
	fExtends[2195] = 80;
	fExtends[2196] = 143;
	fExtends[2197] = 123;
	fExtends[2198] = 101;
	fExtends[2199] = 144;
	fExtends[2200] = 122;
	fExtends[2201] = 101;
	fExtends[2202] = 80;
	fExtends[2203] = 59;
	fExtends[2204] = 59;
	fExtends[2205] = 59;
	fExtends[2206] = 59;
	fExtends[2207] = 59;
	fExtends[2208] = 156;
	fExtends[2209] = 156;
	fExtends[2210] = 157;
	fExtends[2211] = 157;
	fExtends[2212] = 156;
	fExtends[2213] = 157;
	fExtends[2214] = 157;
	fExtends[2215] = 156;
	fExtends[2216] = 156;
	fExtends[2217] = 157;
	fExtends[2218] = 157;
	fExtends[2219] = 156;
	fExtends[2220] = 156;
	fExtends[2221] = 156;
	fExtends[2222] = 156;
	fExtends[2223] = 156;
	fExtends[2224] = 156;
	fExtends[2225] = 156;
	fExtends[2226] = 156;
	fExtends[2227] = 156;
	fExtends[2228] = 156;
	fExtends[2229] = 157;
	fExtends[2230] = 156;
	fExtends[2231] = 156;
	fExtends[2232] = 156;
	fExtends[2233] = 156;
	fExtends[2234] = 158;
	fExtends[2235] = 156;
	fExtends[2236] = 157;
	fExtends[2237] = 157;
	fExtends[2238] = 156;
	fExtends[2239] = 156;
	fExtends[2240] = 156;
	fExtends[2241] = 128;
	fExtends[2242] = 128;
	fExtends[2243] = 128;
	fExtends[2244] = 128;
	fExtends[2245] = 128;
	fExtends[2246] = 156;
	fExtends[2247] = 156;
	fExtends[2248] = 156;
	fExtends[2249] = 128;
	fExtends[2250] = 25;
	fExtends[2251] = 46;
	fExtends[2252] = 46;
	fExtends[2253] = 78;
	fExtends[2254] = 92;
	fExtends[2255] = 126;
	fExtends[2256] = 154;
	fExtends[2257] = 188;
	fExtends[2258] = 38;
	fExtends[2259] = 39;
	fExtends[2260] = 39;
	fExtends[2261] = 39;
	fExtends[2262] = 38;
	fExtends[2263] = 59;
	fExtends[2264] = 59;
	fExtends[2265] = 59;
	fExtends[2266] = 59;
	fExtends[2267] = 59;
	fExtends[2268] = 59;
	fExtends[2269] = 59;
	fExtends[2270] = 59;
	fExtends[2271] = 59;
	fExtends[2272] = 0;
	fExtends[2273] = 0;
	fExtends[2274] = 0;
	fExtends[2275] = 0;
	fExtends[2276] = 0;
	fExtends[2277] = 0;
	fExtends[2278] = 0;
	fExtends[2279] = 0;
	fExtends[2280] = 59;
	fExtends[2281] = 59;
	fExtends[2282] = 59;
	fExtends[2283] = 59;
	fExtends[2284] = 59;
	fExtends[2285] = 59;
	fExtends[2286] = 59;
	fExtends[2287] = 59;
	fExtends[2288] = 100;
	fExtends[2289] = 150;
	fExtends[2290] = 50;
	fExtends[2291] = 3;
	fExtends[2292] = 3;
	fExtends[2293] = 3;
	fExtends[2294] = 24;
	fExtends[2295] = 18;
	fExtends[2296] = 29;
	fExtends[2297] = 59;
	fExtends[2298] = 59;
	fExtends[2299] = 59;
	fExtends[2300] = 59;
	fExtends[2301] = 59;
	fExtends[2302] = 59;
	fExtends[2303] = 59;
	fExtends[2304] = 79;
	fExtends[2305] = 68;
	fExtends[2306] = 53;
	fExtends[2307] = 60;
	fExtends[2308] = 98;
	fExtends[2309] = 130;
	fExtends[2310] = 27;
	fExtends[2311] = 49;
	fExtends[2312] = 49;
	fExtends[2313] = 49;
	fExtends[2314] = 49;
	fExtends[2315] = 49;
	fExtends[2316] = 59;
	fExtends[2317] = 59;
	fExtends[2318] = 59;
	fExtends[2319] = 59;
	fExtends[2320] = 106;
	fExtends[2321] = 106;
	fExtends[2322] = 106;
	fExtends[2323] = 106;
	fExtends[2324] = 99;
	fExtends[2325] = 99;
	fExtends[2326] = 98;
	fExtends[2327] = 99;
	fExtends[2328] = 99;
	fExtends[2329] = 100;
	fExtends[2330] = 99;
	fExtends[2331] = 109;
	fExtends[2332] = 149;
	fExtends[2333] = 108;
	fExtends[2334] = 108;
	fExtends[2335] = 108;
	fExtends[2336] = 25;
	fExtends[2337] = 53;
	fExtends[2338] = 72;
	fExtends[2339] = 53;
	fExtends[2340] = 65;
	fExtends[2341] = 10;
	fExtends[2342] = 50;
	fExtends[2343] = 66;
	fExtends[2344] = 59;
	fExtends[2345] = 76;
	fExtends[2346] = 44;
	fExtends[2347] = 44;
	fExtends[2348] = 210;
	fExtends[2349] = 210;
	fExtends[2350] = 210;
	fExtends[2351] = 210;
	fExtends[2352] = 156;
	fExtends[2353] = 156;
	fExtends[2354] = 157;
	fExtends[2355] = 125;
	fExtends[2356] = 66;
	fExtends[2357] = 66;
	fExtends[2358] = 66;
	fExtends[2359] = 65;
	fExtends[2360] = 56;
	fExtends[2361] = 57;
	fExtends[2362] = 56;
	fExtends[2363] = 56;
	fExtends[2364] = 64;
	fExtends[2365] = 64;
	fExtends[2366] = 6;
	fExtends[2367] = 6;
	fExtends[2368] = 0;
	fExtends[2369] = 44;
	fExtends[2370] = 44;
	fExtends[2371] = 6;
	fExtends[2372] = 6;
	fExtends[2373] = 46;
	fExtends[2374] = 46;
	fExtends[2375] = 44;
	fExtends[2376] = 44;
	fExtends[2377] = 44;
	fExtends[2378] = 44;
	fExtends[2379] = 44;
	fExtends[2380] = 44;
	fExtends[2381] = 59;
	fExtends[2382] = 59;
	fExtends[2383] = 59;
	fExtends[2384] = 156;
	fExtends[2385] = 66;
	fExtends[2386] = 66;
	fExtends[2387] = 56;
	fExtends[2388] = 56;
	fExtends[2389] = 69;
	fExtends[2390] = 66;
	fExtends[2391] = 57;
	fExtends[2392] = 56;
	fExtends[2393] = 56;
	fExtends[2394] = 56;
	fExtends[2395] = 102;
	fExtends[2396] = 125;
	fExtends[2397] = 65;
	fExtends[2398] = 65;
	fExtends[2399] = 64;
	fExtends[2400] = 64;
	fExtends[2401] = 70;
	fExtends[2402] = 59;
	fExtends[2403] = 59;
	fExtends[2404] = 59;
	fExtends[2405] = 59;
	fExtends[2406] = 59;
	fExtends[2407] = 59;
	fExtends[2408] = 59;
	fExtends[2409] = 59;
	fExtends[2410] = 59;
	fExtends[2411] = 59;
	fExtends[2412] = 59;
	fExtends[2413] = 59;
	fExtends[2414] = 59;
	fExtends[2415] = 59;
	fExtends[2416] = 156;
	fExtends[2417] = 156;
	fExtends[2418] = 157;
	fExtends[2419] = 156;
	fExtends[2420] = 156;
	fExtends[2421] = 156;
	fExtends[2422] = 157;
	fExtends[2423] = 156;
	fExtends[2424] = 156;
	fExtends[2425] = 156;
	fExtends[2426] = 157;
	fExtends[2427] = 156;
	fExtends[2428] = 156;
	fExtends[2429] = 156;
	fExtends[2430] = 157;
	fExtends[2431] = 156;
	fExtends[2432] = 156;
	fExtends[2433] = 156;
	fExtends[2434] = 157;
	fExtends[2435] = 156;
	fExtends[2436] = 156;
	fExtends[2437] = 156;
	fExtends[2438] = 157;
	fExtends[2439] = 156;
	fExtends[2440] = 156;
	fExtends[2441] = 156;
	fExtends[2442] = 157;
	fExtends[2443] = 156;
	fExtends[2444] = 156;
	fExtends[2445] = 156;
	fExtends[2446] = 157;
	fExtends[2447] = 156;
	fExtends[2448] = 32;
	fExtends[2449] = 30;
	fExtends[2450] = 32;
	fExtends[2451] = 18;
	fExtends[2452] = 31;
	fExtends[2453] = 31;
	fExtends[2454] = 32;
	fExtends[2455] = 32;
	fExtends[2456] = 32;
	fExtends[2457] = 53;
	fExtends[2458] = 53;
	fExtends[2459] = 31;
	fExtends[2460] = 31;
	fExtends[2461] = 31;
	fExtends[2462] = 31;
	fExtends[2463] = 25;
	fExtends[2464] = 30;
	fExtends[2465] = 18;
	fExtends[2466] = 59;
	fExtends[2467] = 59;
	fExtends[2468] = 59;
	fExtends[2469] = 59;
	fExtends[2470] = 59;
	fExtends[2471] = 59;
	fExtends[2472] = 59;
	fExtends[2473] = 59;
	fExtends[2474] = 59;
	fExtends[2475] = 59;
	fExtends[2476] = 59;
	fExtends[2477] = 59;
	fExtends[2478] = 59;
	fExtends[2479] = 59;
	fExtends[2480] = 28;
	fExtends[2481] = 0;
	fExtends[2482] = 3;
	fExtends[2483] = 3;
	fExtends[2484] = 3;
	fExtends[2485] = 3;
	fExtends[2486] = 3;
	fExtends[2487] = 3;
	fExtends[2488] = 3;
	fExtends[2489] = 93;
	fExtends[2490] = 116;
	fExtends[2491] = 116;
	fExtends[2492] = 116;
	fExtends[2493] = 3;
	fExtends[2494] = 3;
	fExtends[2495] = 3;
	fExtends[2496] = 3;
	fExtends[2497] = 3;
	fExtends[2498] = 32;
	fExtends[2499] = 33;
	fExtends[2500] = 34;
	fExtends[2501] = 36;
	fExtends[2502] = 59;
	fExtends[2503] = 59;
	fExtends[2504] = 59;
	fExtends[2505] = 59;
	fExtends[2506] = 59;
	fExtends[2507] = 59;
	fExtends[2508] = 59;
	fExtends[2509] = 59;
	fExtends[2510] = 59;
	fExtends[2511] = 59;
	fExtends[2512] = 6;
	fExtends[2513] = 6;
	fExtends[2514] = 19;
	fExtends[2515] = 19;
	fExtends[2516] = 19;
	fExtends[2517] = 19;
	fExtends[2518] = 15;
	fExtends[2519] = 15;
	fExtends[2520] = 33;
	fExtends[2521] = 13;
	fExtends[2522] = 59;
	fExtends[2523] = 59;
	fExtends[2524] = 59;
	fExtends[2525] = 59;
	fExtends[2526] = 59;
	fExtends[2527] = 59;
	fExtends[2528] = 41;
	fExtends[2529] = 37;
	fExtends[2530] = 20;
	fExtends[2531] = 86;
	fExtends[2532] = 45;
	fExtends[2533] = 34;
	fExtends[2534] = 59;
	fExtends[2535] = 59;
	fExtends[2536] = 59;
	fExtends[2537] = 59;
	fExtends[2538] = 59;
	fExtends[2539] = 59;
	fExtends[2540] = 59;
	fExtends[2541] = 59;
	fExtends[2542] = 59;
	fExtends[2543] = 59;
	fExtends[2544] = 29;
	fExtends[2545] = 10;
	fExtends[2546] = 10;
	fExtends[2547] = 10;
	fExtends[2548] = 10;
	fExtends[2549] = 10;
	fExtends[2550] = 28;
	fExtends[2551] = 28;
	fExtends[2552] = 28;
	fExtends[2553] = 59;
	fExtends[2554] = 59;
	fExtends[2555] = 59;
	fExtends[2556] = 59;
	fExtends[2557] = 59;
	fExtends[2558] = 59;
	fExtends[2559] = 59;
	fExtends[2560] = 57;
	fExtends[2561] = 57;
	fExtends[2562] = 123;
	fExtends[2563] = 123;
	fExtends[2564] = 12;
	fExtends[2565] = 12;
	fExtends[2566] = 12;
	fExtends[2567] = 12;
	fExtends[2568] = 12;
	fExtends[2569] = 12;
	fExtends[2570] = 112;
	fExtends[2571] = 93;
	fExtends[2572] = 36;
	fExtends[2573] = 36;
	fExtends[2574] = 36;
	fExtends[2575] = 36;
	fExtends[2576] = 60;
	fExtends[2577] = 59;
	fExtends[2578] = 59;
	fExtends[2579] = 59;
	fExtends[2580] = 59;
	fExtends[2581] = 59;
	fExtends[2582] = 59;
	fExtends[2583] = 59;
	fExtends[2584] = 59;
	fExtends[2585] = 59;
	fExtends[2586] = 59;
	fExtends[2587] = 59;
	fExtends[2588] = 59;
	fExtends[2589] = 59;
	fExtends[2590] = 59;
	fExtends[2591] = 59;
	fExtends[2592] = 83;
	fExtends[2593] = 100;
	fExtends[2594] = 66;
	fExtends[2595] = 60;
	fExtends[2596] = 75;
	fExtends[2597] = 55;
	fExtends[2598] = 55;
	fExtends[2599] = 55;
	fExtends[2600] = 81;
	fExtends[2601] = 37;
	fExtends[2602] = 64;
	fExtends[2603] = 59;
	fExtends[2604] = 59;
	fExtends[2605] = 59;
	fExtends[2606] = 59;
	fExtends[2607] = 59;
	fExtends[2608] = 131;
	fExtends[2609] = 121;
	fExtends[2610] = 133;
	fExtends[2611] = 122;
	fExtends[2612] = 131;
	fExtends[2613] = 121;
	fExtends[2614] = 63;
	fExtends[2615] = 122;
	fExtends[2616] = 131;
	fExtends[2617] = 121;
	fExtends[2618] = 54;
	fExtends[2619] = 122;
	fExtends[2620] = 131;
	fExtends[2621] = 121;
	fExtends[2622] = 114;
	fExtends[2623] = 122;
	fExtends[2624] = 151;
	fExtends[2625] = 115;
	fExtends[2626] = 59;
	fExtends[2627] = 59;
	fExtends[2628] = 59;
	fExtends[2629] = 59;
	fExtends[2630] = 59;
	fExtends[2631] = 59;
	fExtends[2632] = 59;
	fExtends[2633] = 59;
	fExtends[2634] = 59;
	fExtends[2635] = 59;
	fExtends[2636] = 59;
	fExtends[2637] = 59;
	fExtends[2638] = 59;
	fExtends[2639] = 59;
	fExtends[2640] = 51;
	fExtends[2641] = 31;
	fExtends[2642] = 48;
	fExtends[2643] = 49;
	fExtends[2644] = 45;
	fExtends[2645] = 47;
	fExtends[2646] = 55;
	fExtends[2647] = 45;
	fExtends[2648] = 46;
	fExtends[2649] = 46;
	fExtends[2650] = 46;
	fExtends[2651] = 47;
	fExtends[2652] = 47;
	fExtends[2653] = 47;
	fExtends[2654] = 47;
	fExtends[2655] = 47;
	fExtends[2656] = 46;
	fExtends[2657] = 47;
	fExtends[2658] = 47;
	fExtends[2659] = 72;
	fExtends[2660] = 43;
	fExtends[2661] = 32;
	fExtends[2662] = 46;
	fExtends[2663] = 59;
	fExtends[2664] = 14;
	fExtends[2665] = 14;
	fExtends[2666] = 17;
	fExtends[2667] = 18;
	fExtends[2668] = 27;
	fExtends[2669] = 82;
	fExtends[2670] = 82;
	fExtends[2671] = 39;
	fExtends[2672] = 101;
	fExtends[2673] = 57;
	fExtends[2674] = 103;
	fExtends[2675] = 103;
	fExtends[2676] = 106;
	fExtends[2677] = 104;
	fExtends[2678] = 102;
	fExtends[2679] = 103;
	fExtends[2680] = 103;
	fExtends[2681] = 102;
	fExtends[2682] = 107;
	fExtends[2683] = 56;
	fExtends[2684] = 108;
	fExtends[2685] = 162;
	fExtends[2686] = 124;
	fExtends[2687] = 124;
	fExtends[2688] = 102;
	fExtends[2689] = 166;
	fExtends[2690] = 166;
	fExtends[2691] = 141;
	fExtends[2692] = 100;
	fExtends[2693] = 122;
	fExtends[2694] = 91;
	fExtends[2695] = 113;
	fExtends[2696] = 102;
	fExtends[2697] = 123;
	fExtends[2698] = 93;
	fExtends[2699] = 122;
	fExtends[2700] = 56;
	fExtends[2701] = 131;
	fExtends[2702] = 103;
	fExtends[2703] = 37;
	fExtends[2704] = 37;
	fExtends[2705] = 45;
	fExtends[2706] = 45;
	fExtends[2707] = 88;
	fExtends[2708] = 88;
	fExtends[2709] = 93;
	fExtends[2710] = 108;
	fExtends[2711] = 77;
	fExtends[2712] = 77;
	fExtends[2713] = 104;
	fExtends[2714] = 36;
	fExtends[2715] = 45;
	fExtends[2716] = 124;
	fExtends[2717] = 96;
	fExtends[2718] = 99;
	fExtends[2719] = 44;
	fExtends[2720] = 27;
	fExtends[2721] = 28;
	fExtends[2722] = 34;
	fExtends[2723] = 40;
	fExtends[2724] = 47;
	fExtends[2725] = 56;
	fExtends[2726] = 68;
	fExtends[2727] = 80;
	fExtends[2728] = 99;
	fExtends[2729] = 51;
	fExtends[2730] = 51;
	fExtends[2731] = 106;
	fExtends[2732] = 89;
	fExtends[2733] = 103;
	fExtends[2734] = 103;
	fExtends[2735] = 52;
	fExtends[2736] = 30;
	fExtends[2737] = 41;
	fExtends[2738] = 30;
	fExtends[2739] = 41;
	fExtends[2740] = 100;
	fExtends[2741] = 150;
	fExtends[2742] = 200;
	fExtends[2743] = 100;
	fExtends[2744] = 150;
	fExtends[2745] = 200;
	fExtends[2746] = 100;
	fExtends[2747] = 150;
	fExtends[2748] = 200;
	fExtends[2749] = 61;
	fExtends[2750] = 25;
	fExtends[2751] = 25;
	fExtends[2752] = 54;
	fExtends[2753] = 54;
	fExtends[2754] = 109;
	fExtends[2755] = 109;
	fExtends[2756] = 96;
	fExtends[2757] = 230;
	fExtends[2758] = 148;
	fExtends[2759] = 117;
	fExtends[2760] = 86;
	fExtends[2761] = 92;
	fExtends[2762] = 67;
	fExtends[2763] = 32;
	fExtends[2764] = 90;
	fExtends[2765] = 24;
	fExtends[2766] = 27;
	fExtends[2767] = 29;
	fExtends[2768] = 35;
	fExtends[2769] = 42;
	fExtends[2770] = 49;
	fExtends[2771] = 57;
	fExtends[2772] = 27;
	fExtends[2773] = 31;
	fExtends[2774] = 36;
	fExtends[2775] = 43;
	fExtends[2776] = 50;
	fExtends[2777] = 58;
	fExtends[2778] = 75;
	fExtends[2779] = 37;
	fExtends[2780] = 41;
	fExtends[2781] = 48;
	fExtends[2782] = 59;
	fExtends[2783] = 69;
	fExtends[2784] = 84;
	fExtends[2785] = 100;
	fExtends[2786] = 52;
	fExtends[2787] = 63;
	fExtends[2788] = 75;
	fExtends[2789] = 88;
	fExtends[2790] = 108;
	fExtends[2791] = 130;
	fExtends[2792] = 153;
	fExtends[2793] = 77;
	fExtends[2794] = 94;
	fExtends[2795] = 112;
	fExtends[2796] = 133;
	fExtends[2797] = 158;
	fExtends[2798] = 191;
	fExtends[2799] = 227;
	fExtends[2800] = 399;
	fExtends[2801] = 399;
	fExtends[2802] = 399;
	fExtends[2803] = 399;
	fExtends[2804] = 237;
	fExtends[2805] = 222;
	fExtends[2806] = 207;
	fExtends[2807] = 196;
	fExtends[2808] = 181;
	fExtends[2809] = 166;
	fExtends[2810] = 151;
	fExtends[2811] = 136;
	fExtends[2812] = 121;
	fExtends[2813] = 106;
	fExtends[2814] = 91;
	fExtends[2815] = 76;
	fExtends[2816] = 61;
	fExtends[2817] = 46;
	fExtends[2818] = 31;
	fExtends[2819] = 6;
	fExtends[2820] = 59;
	fExtends[2821] = 59;
	fExtends[2822] = 59;
	fExtends[2823] = 59;
	fExtends[2824] = 59;
	fExtends[2825] = 59;
	fExtends[2826] = 59;
	fExtends[2827] = 59;
	fExtends[2828] = 59;
	fExtends[2829] = 59;
	fExtends[2830] = 59;
	fExtends[2831] = 59;
	fExtends[2832] = 110;
	fExtends[2833] = 176;
	fExtends[2834] = 176;
	fExtends[2835] = 200;
	fExtends[2836] = 200;
	fExtends[2837] = 172;
	fExtends[2838] = 89;
	fExtends[2839] = 189;
	fExtends[2840] = 201;
	fExtends[2841] = 200;
	fExtends[2842] = 133;
	fExtends[2843] = 200;
	fExtends[2844] = 94;
	fExtends[2845] = 107;
	fExtends[2846] = 99;
	fExtends[2847] = 180;
	fExtends[2848] = 180;
	fExtends[2849] = 196;
	fExtends[2850] = 196;
	fExtends[2851] = 150;
	fExtends[2852] = 132;
	fExtends[2853] = 229;
	fExtends[2854] = 219;
	fExtends[2855] = 192;
	fExtends[2856] = 110;
	fExtends[2857] = 114;
	fExtends[2858] = 135;
	fExtends[2859] = 107;
	fExtends[2860] = 5;
	fExtends[2861] = 42;
	fExtends[2862] = 47;
	fExtends[2863] = 47;
	fExtends[2864] = 47;
	fExtends[2865] = 47;
	fExtends[2866] = 47;
	fExtends[2867] = 47;
	fExtends[2868] = 107;
	fExtends[2869] = 107;
	fExtends[2870] = 110;
	fExtends[2871] = 110;
	fExtends[2872] = 110;
	fExtends[2873] = 110;
	fExtends[2874] = 110;
	fExtends[2875] = 110;
	fExtends[2876] = 135;
	fExtends[2877] = 200;
	fExtends[2878] = 200;
	fExtends[2879] = 200;
	fExtends[2880] = 200;
	fExtends[2881] = 200;
	fExtends[2882] = 200;
	fExtends[2883] = 200;
	fExtends[2884] = 200;
	fExtends[2885] = 200;
	fExtends[2886] = 200;
	fExtends[2887] = 164;
	fExtends[2888] = 164;
	fExtends[2889] = 200;
	fExtends[2890] = 200;
	fExtends[2891] = 163;
	fExtends[2892] = 163;
	fExtends[2893] = 213;
	fExtends[2894] = 213;
	fExtends[2895] = 109;
	fExtends[2896] = 109;
	fExtends[2897] = 59;
	fExtends[2898] = 59;
	fExtends[2899] = 59;
	fExtends[2900] = 59;
	fExtends[2901] = 59;
	fExtends[2902] = 59;
	fExtends[2903] = 59;
	fExtends[2904] = 59;
	fExtends[2905] = 59;
	fExtends[2906] = 59;
	fExtends[2907] = 59;
	fExtends[2908] = 59;
	fExtends[2909] = 59;
	fExtends[2910] = 59;
	fExtends[2911] = 59;
	fExtends[2912] = 46;
	fExtends[2913] = 77;
	fExtends[2914] = 105;
	fExtends[2915] = 77;
	fExtends[2916] = 46;
	fExtends[2917] = 77;
	fExtends[2918] = 105;
	fExtends[2919] = 77;
	fExtends[2920] = 46;
	fExtends[2921] = 77;
	fExtends[2922] = 105;
	fExtends[2923] = 77;
	fExtends[2924] = 46;
	fExtends[2925] = 77;
	fExtends[2926] = 105;
	fExtends[2927] = 77;
	fExtends[2928] = 42;
	fExtends[2929] = 77;
	fExtends[2930] = 106;
	fExtends[2931] = 77;
	fExtends[2932] = 42;
	fExtends[2933] = 77;
	fExtends[2934] = 106;
	fExtends[2935] = 77;
	fExtends[2936] = 46;
	fExtends[2937] = 58;
	fExtends[2938] = 60;
	fExtends[2939] = 58;
	fExtends[2940] = 46;
	fExtends[2941] = 58;
	fExtends[2942] = 60;
	fExtends[2943] = 58;
	fExtends[2944] = 46;
	fExtends[2945] = 58;
	fExtends[2946] = 60;
	fExtends[2947] = 58;
	fExtends[2948] = 46;
	fExtends[2949] = 58;
	fExtends[2950] = 60;
	fExtends[2951] = 58;
	fExtends[2952] = 42;
	fExtends[2953] = 52;
	fExtends[2954] = 56;
	fExtends[2955] = 52;
	fExtends[2956] = 42;
	fExtends[2957] = 52;
	fExtends[2958] = 56;
	fExtends[2959] = 52;
	fExtends[2960] = 0;
	fExtends[2961] = 0;
	fExtends[2962] = 0;
	fExtends[2963] = 0;
	fExtends[2964] = 0;
	fExtends[2965] = 0;
	fExtends[2966] = 0;
	fExtends[2967] = 0;
	fExtends[2968] = 0;
	fExtends[2969] = 0;
	fExtends[2970] = 0;
	fExtends[2971] = 0;
	fExtends[2972] = 0;
	fExtends[2973] = 0;
	fExtends[2974] = 0;
	fExtends[2975] = 0;
	fExtends[2976] = 100;
	fExtends[2977] = 150;
	fExtends[2978] = 50;
	fExtends[2979] = 75;
	fExtends[2980] = 75;
	fExtends[2981] = 46;
	fExtends[2982] = 30;
	fExtends[2983] = 7;
	fExtends[2984] = 46;
	fExtends[2985] = 46;
	fExtends[2986] = 46;
	fExtends[2987] = 46;
	fExtends[2988] = 46;
	fExtends[2989] = 6;
	fExtends[2990] = 14;
	fExtends[2991] = 41;
	fExtends[2992] = 68;
	fExtends[2993] = 59;
	fExtends[2994] = 59;
	fExtends[2995] = 59;
	fExtends[2996] = 59;
	fExtends[2997] = 59;
	fExtends[2998] = 59;
	fExtends[2999] = 59;
	fExtends[3000] = 59;
	fExtends[3001] = 59;
	fExtends[3002] = 59;
	fExtends[3003] = 59;
	fExtends[3004] = 59;
	fExtends[3005] = 59;
	fExtends[3006] = 59;
	fExtends[3007] = 59;
	fExtends[3008] = 55;
	fExtends[3009] = 44;
	fExtends[3010] = 58;
	fExtends[3011] = 86;
	fExtends[3012] = 63;
	fExtends[3013] = 50;
	fExtends[3014] = 81;
	fExtends[3015] = 47;
	fExtends[3016] = 25;
	fExtends[3017] = 43;
	fExtends[3018] = 26;
	fExtends[3019] = 82;
	fExtends[3020] = 64;
	fExtends[3021] = 55;
	fExtends[3022] = 92;
	fExtends[3023] = 114;
	fExtends[3024] = 136;
	fExtends[3025] = 54;
	fExtends[3026] = 79;
	fExtends[3027] = 37;
	fExtends[3028] = 29;
	fExtends[3029] = 26;
	fExtends[3030] = 59;
	fExtends[3031] = 59;
	fExtends[3032] = 59;
	fExtends[3033] = 59;
	fExtends[3034] = 59;
	fExtends[3035] = 59;
	fExtends[3036] = 59;
	fExtends[3037] = 59;
	fExtends[3038] = 59;
	fExtends[3039] = 59;
	fExtends[3040] = 51;
	fExtends[3041] = 28;
	fExtends[3042] = 48;
	fExtends[3043] = 44;
	fExtends[3044] = 47;
	fExtends[3045] = 43;
	fExtends[3046] = 47;
	fExtends[3047] = 48;
	fExtends[3048] = 46;
	fExtends[3049] = 47;
	fExtends[3050] = 225;
	fExtends[3051] = 225;
	fExtends[3052] = 195;
	fExtends[3053] = 196;
	fExtends[3054] = 195;
	fExtends[3055] = 133;
	fExtends[3056] = 158;
	fExtends[3057] = 95;
	fExtends[3058] = 41;
	fExtends[3059] = 61;
	fExtends[3060] = 135;
	fExtends[3061] = 70;
	fExtends[3062] = 79;
	fExtends[3063] = 59;
	fExtends[3064] = 59;
	fExtends[3065] = 59;
	fExtends[3066] = 59;
	fExtends[3067] = 59;
	fExtends[3068] = 59;
	fExtends[3069] = 59;
	fExtends[3070] = 59;
	fExtends[3071] = 59;
	fExtends[3072] = 56;
	fExtends[3073] = 60;
	fExtends[3074] = 40;
	fExtends[3075] = 58;
	fExtends[3076] = 43;
	fExtends[3077] = 49;
	fExtends[3078] = 69;
	fExtends[3079] = 61;
	fExtends[3080] = 27;
	fExtends[3081] = 50;
	fExtends[3082] = 34;
	fExtends[3083] = 97;
	fExtends[3084] = 63;
	fExtends[3085] = 63;
	fExtends[3086] = 66;
	fExtends[3087] = 63;
	fExtends[3088] = 48;
	fExtends[3089] = 49;
	fExtends[3090] = 45;
	fExtends[3091] = 67;
	fExtends[3092] = 51;
	fExtends[3093] = 61;
	fExtends[3094] = 43;
	fExtends[3095] = 96;
	fExtends[3096] = 112;
	fExtends[3097] = 83;
	fExtends[3098] = 112;
	fExtends[3099] = 101;
	fExtends[3100] = 80;
	fExtends[3101] = 109;
	fExtends[3102] = 95;
	fExtends[3103] = 105;
	fExtends[3104] = 105;
	fExtends[3105] = 84;
	fExtends[3106] = 144;
	fExtends[3107] = 121;
	fExtends[3108] = 59;
	fExtends[3109] = 59;
	fExtends[3110] = 59;
	fExtends[3111] = 59;
	fExtends[3112] = 59;
	fExtends[3113] = 59;
	fExtends[3114] = 59;
	fExtends[3115] = 59;
	fExtends[3116] = 59;
	fExtends[3117] = 59;
	fExtends[3118] = 59;
	fExtends[3119] = 59;
	fExtends[3120] = 67;
	fExtends[3121] = 71;
	fExtends[3122] = 232;
	fExtends[3123] = 44;
	fExtends[3124] = 60;
	fExtends[3125] = 48;
	fExtends[3126] = 53;
	fExtends[3127] = 47;
	fExtends[3128] = 47;
	fExtends[3129] = 46;
	fExtends[3130] = 57;
	fExtends[3131] = 100;
	fExtends[3132] = 40;
	fExtends[3133] = 75;
	fExtends[3134] = 40;
	fExtends[3135] = 59;
	fExtends[3136] = 197;
	fExtends[3137] = 220;
	fExtends[3138] = 250;
	fExtends[3139] = 207;
	fExtends[3140] = 232;
	fExtends[3141] = 221;
	fExtends[3142] = 210;
	fExtends[3143] = 59;
	fExtends[3144] = 59;
	fExtends[3145] = 59;
	fExtends[3146] = 59;
	fExtends[3147] = 59;
	fExtends[3148] = 59;
	fExtends[3149] = 59;
	fExtends[3150] = 59;
	fExtends[3151] = 59;
	fExtends[3152] = 77;
	fExtends[3153] = 77;
	fExtends[3154] = 79;
	fExtends[3155] = 79;
	fExtends[3156] = 64;
	fExtends[3157] = 88;
	fExtends[3158] = 108;
	fExtends[3159] = 108;
	fExtends[3160] = 45;
	fExtends[3161] = 77;
	fExtends[3162] = 77;
	fExtends[3163] = 59;
	fExtends[3164] = 59;
	fExtends[3165] = 59;
	fExtends[3166] = 59;
	fExtends[3167] = 59;
	fExtends[3168] = 217;
	fExtends[3169] = 226;
	fExtends[3170] = 233;
	fExtends[3171] = 165;
	fExtends[3172] = 165;
	fExtends[3173] = 59;
	fExtends[3174] = 59;
	fExtends[3175] = 59;
	fExtends[3176] = 59;
	fExtends[3177] = 59;
	fExtends[3178] = 59;
	fExtends[3179] = 59;
	fExtends[3180] = 59;
	fExtends[3181] = 59;
	fExtends[3182] = 59;
	fExtends[3183] = 59;
	fExtends[3184] = 59;
	fExtends[3185] = 59;
	fExtends[3186] = 59;
	fExtends[3187] = 59;
	fExtends[3188] = 59;
	fExtends[3189] = 59;
	fExtends[3190] = 59;
	fExtends[3191] = 59;
	fExtends[3192] = 59;
	fExtends[3193] = 59;
	fExtends[3194] = 59;
	fExtends[3195] = 59;
	fExtends[3196] = 59;
	fExtends[3197] = 59;
	fExtends[3198] = 59;
	fExtends[3199] = 59;
	fExtends[3200] = 24;
	fExtends[3201] = 24;
	fExtends[3202] = 24;
	fExtends[3203] = 24;
	fExtends[3204] = 65;
	fExtends[3205] = 89;
	fExtends[3206] = 80;
	fExtends[3207] = 59;
	fExtends[3208] = 59;
	fExtends[3209] = 59;
	fExtends[3210] = 59;
	fExtends[3211] = 59;
	fExtends[3212] = 59;
	fExtends[3213] = 59;
	fExtends[3214] = 59;
	fExtends[3215] = 59;
	fExtends[3216] = 151;
	fExtends[3217] = 48;
	fExtends[3218] = 48;
	fExtends[3219] = 40;
	fExtends[3220] = 40;
	fExtends[3221] = 76;
	fExtends[3222] = 76;
	fExtends[3223] = 43;
	fExtends[3224] = 43;
	fExtends[3225] = 59;
	fExtends[3226] = 59;
	fExtends[3227] = 59;
	fExtends[3228] = 59;
	fExtends[3229] = 59;
	fExtends[3230] = 59;
	fExtends[3231] = 59;
	fExtends[3232] = 131;
	fExtends[3233] = 107;
	fExtends[3234] = 92;
	fExtends[3235] = 68;
	fExtends[3236] = 68;
	fExtends[3237] = 66;
	fExtends[3238] = 66;
	fExtends[3239] = 107;
	fExtends[3240] = 66;
	fExtends[3241] = 104;
	fExtends[3242] = 66;
	fExtends[3243] = 108;
	fExtends[3244] = 66;
	fExtends[3245] = 107;
	fExtends[3246] = 66;
	fExtends[3247] = 107;
	fExtends[3248] = 66;
	fExtends[3249] = 108;
	fExtends[3250] = 66;
	fExtends[3251] = 108;
	fExtends[3252] = 66;
	fExtends[3253] = 108;
	fExtends[3254] = 66;
	fExtends[3255] = 20;
	fExtends[3256] = 59;
	fExtends[3257] = 59;
	fExtends[3258] = 59;
	fExtends[3259] = 59;
	fExtends[3260] = 59;
	fExtends[3261] = 59;
	fExtends[3262] = 59;
	fExtends[3263] = 59;
	fExtends[3264] = 47;
	fExtends[3265] = 59;
	fExtends[3266] = 59;
	fExtends[3267] = 59;
	fExtends[3268] = 59;
	fExtends[3269] = 59;
	fExtends[3270] = 59;
	fExtends[3271] = 59;
	fExtends[3272] = 59;
	fExtends[3273] = 59;
	fExtends[3274] = 59;
	fExtends[3275] = 59;
	fExtends[3276] = 59;
	fExtends[3277] = 59;
	fExtends[3278] = 59;
	fExtends[3279] = 59;
	fExtends[3280] = 120;
	fExtends[3281] = 116;
	fExtends[3282] = 115;
	fExtends[3283] = 116;
	fExtends[3284] = 102;
	fExtends[3285] = 107;
	fExtends[3286] = 117;
	fExtends[3287] = 108;
	fExtends[3288] = 99;
	fExtends[3289] = 92;
	fExtends[3290] = 96;
	fExtends[3291] = 92;
	fExtends[3292] = 95;
	fExtends[3293] = 104;
	fExtends[3294] = 59;
	fExtends[3295] = 59;
	fExtends[3296] = 94;
	fExtends[3297] = 67;
	fExtends[3298] = 89;
	fExtends[3299] = 84;
	fExtends[3300] = 94;
	fExtends[3301] = 81;
	fExtends[3302] = 87;
	fExtends[3303] = 88;
	fExtends[3304] = 87;
	fExtends[3305] = 87;
	fExtends[3306] = 85;
	fExtends[3307] = 83;
	fExtends[3308] = 59;
	fExtends[3309] = 59;
	fExtends[3310] = 59;
	fExtends[3311] = 59;
	fExtends[3312] = 94;
	fExtends[3313] = 67;
	fExtends[3314] = 89;
	fExtends[3315] = 84;
	fExtends[3316] = 94;
	fExtends[3317] = 81;
	fExtends[3318] = 87;
	fExtends[3319] = 88;
	fExtends[3320] = 87;
	fExtends[3321] = 87;
	fExtends[3322] = 85;
	fExtends[3323] = 83;
	fExtends[3324] = 59;
	fExtends[3325] = 59;
	fExtends[3326] = 59;
	fExtends[3327] = 59;
	fExtends[3328] = 147;
	fExtends[3329] = 146;
	fExtends[3330] = 74;
	fExtends[3331] = 62;
	fExtends[3332] = 59;
	fExtends[3333] = 59;
	fExtends[3334] = 59;
	fExtends[3335] = 59;
	fExtends[3336] = 59;
	fExtends[3337] = 59;
	fExtends[3338] = 59;
	fExtends[3339] = 59;
	fExtends[3340] = 59;
	fExtends[3341] = 59;
	fExtends[3342] = 59;
	fExtends[3343] = 59;
	fExtends[3344] = 51;
	fExtends[3345] = 31;
	fExtends[3346] = 48;
	fExtends[3347] = 45;
	fExtends[3348] = 47;
	fExtends[3349] = 45;
	fExtends[3350] = 49;
	fExtends[3351] = 54;
	fExtends[3352] = 30;
	fExtends[3353] = 33;
	fExtends[3354] = 83;
	fExtends[3355] = 53;
	fExtends[3356] = 44;
	fExtends[3357] = 57;
	fExtends[3358] = 40;
	fExtends[3359] = 46;
	fExtends[3360] = 56;
	fExtends[3361] = 56;
	fExtends[3362] = 40;
	fExtends[3363] = 31;
	fExtends[3364] = 47;
	fExtends[3365] = 48;
	fExtends[3366] = 46;
	fExtends[3367] = 47;
	fExtends[3368] = 18;
	fExtends[3369] = 18;
	fExtends[3370] = 13;
	fExtends[3371] = 13;
	fExtends[3372] = 29;
	fExtends[3373] = 29;
	fExtends[3374] = 33;
	fExtends[3375] = 59;
	fExtends[3376] = 83;
	fExtends[3377] = 56;
	fExtends[3378] = 45;
	fExtends[3379] = 57;
	fExtends[3380] = 34;
	fExtends[3381] = 33;
	fExtends[3382] = 50;
	fExtends[3383] = 63;
	fExtends[3384] = 50;
	fExtends[3385] = 59;
	fExtends[3386] = 59;
	fExtends[3387] = 59;
	fExtends[3388] = 59;
	fExtends[3389] = 59;
	fExtends[3390] = 59;
	fExtends[3391] = 59;
	fExtends[3392] = 142;
	fExtends[3393] = 142;
	fExtends[3394] = 176;
	fExtends[3395] = 176;
	fExtends[3396] = 192;
	fExtends[3397] = 192;
	fExtends[3398] = 192;
	fExtends[3399] = 192;
	fExtends[3400] = 59;
	fExtends[3401] = 59;
	fExtends[3402] = 59;
	fExtends[3403] = 59;
	fExtends[3404] = 59;
	fExtends[3405] = 59;
	fExtends[3406] = 59;
	fExtends[3407] = 59;
	fExtends[3408] = 20;
	fExtends[3409] = 20;
	fExtends[3410] = 45;
	fExtends[3411] = 46;
	fExtends[3412] = 34;
	fExtends[3413] = 34;
	fExtends[3414] = 50;
	fExtends[3415] = 50;
	fExtends[3416] = 33;
	fExtends[3417] = 57;
	fExtends[3418] = 50;
	fExtends[3419] = 45;
	fExtends[3420] = 45;
	fExtends[3421] = 50;
	fExtends[3422] = 50;
	fExtends[3423] = 59;
	fExtends[3424] = 57;
	fExtends[3425] = 37;
	fExtends[3426] = 62;
	fExtends[3427] = 89;
	fExtends[3428] = 105;
	fExtends[3429] = 117;
	fExtends[3430] = 153;
	fExtends[3431] = 59;
	fExtends[3432] = 59;
	fExtends[3433] = 59;
	fExtends[3434] = 59;
	fExtends[3435] = 59;
	fExtends[3436] = 59;
	fExtends[3437] = 59;
	fExtends[3438] = 59;
	fExtends[3439] = 59;
	fExtends[3440] = 129;
	fExtends[3441] = 59;
	fExtends[3442] = 59;
	fExtends[3443] = 59;
	fExtends[3444] = 59;
	fExtends[3445] = 59;
	fExtends[3446] = 59;
	fExtends[3447] = 59;
	fExtends[3448] = 59;
	fExtends[3449] = 59;
	fExtends[3450] = 59;
	fExtends[3451] = 59;
	fExtends[3452] = 59;
	fExtends[3453] = 59;
	fExtends[3454] = 59;
	fExtends[3455] = 59;
	fExtends[3456] = 46;
	fExtends[3457] = 39;
	fExtends[3458] = 48;
	fExtends[3459] = 45;
	fExtends[3460] = 47;
	fExtends[3461] = 49;
	fExtends[3462] = 47;
	fExtends[3463] = 48;
	fExtends[3464] = 48;
	fExtends[3465] = 47;
	fExtends[3466] = 21;
	fExtends[3467] = 20;
	fExtends[3468] = 22;
	fExtends[3469] = 21;
	fExtends[3470] = 59;
	fExtends[3471] = 59;
	fExtends[3472] = 59;
	fExtends[3473] = 59;
	fExtends[3474] = 59;
	fExtends[3475] = 59;
	fExtends[3476] = 59;
	fExtends[3477] = 59;
	fExtends[3478] = 59;
	fExtends[3479] = 59;
	fExtends[3480] = 59;
	fExtends[3481] = 59;
	fExtends[3482] = 59;
	fExtends[3483] = 59;
	fExtends[3484] = 59;
	fExtends[3485] = 59;
	fExtends[3486] = 59;
	fExtends[3487] = 59;
	fExtends[3488] = 56;
	fExtends[3489] = 56;
	fExtends[3490] = 56;
	fExtends[3491] = 81;
	fExtends[3492] = 49;
	fExtends[3493] = 104;
	fExtends[3494] = 56;
	fExtends[3495] = 56;
	fExtends[3496] = 54;
	fExtends[3497] = 42;
	fExtends[3498] = 39;
	fExtends[3499] = 39;
	fExtends[3500] = 27;
	fExtends[3501] = 27;
	fExtends[3502] = 45;
	fExtends[3503] = 112;
	fExtends[3504] = 180;
	fExtends[3505] = 111;
	fExtends[3506] = 131;
	fExtends[3507] = 173;
	fExtends[3508] = 173;
	fExtends[3509] = 131;
	fExtends[3510] = 61;
	fExtends[3511] = 50;
	fExtends[3512] = 57;
	fExtends[3513] = 67;
	fExtends[3514] = 58;
	fExtends[3515] = 97;
	fExtends[3516] = 92;
	fExtends[3517] = 86;
	fExtends[3518] = 86;
	fExtends[3519] = 86;
	fExtends[3520] = 86;
	fExtends[3521] = 97;
	fExtends[3522] = 97;
	fExtends[3523] = 64;
	fExtends[3524] = 60;
	fExtends[3525] = 64;
	fExtends[3526] = 82;
	fExtends[3527] = 79;
	fExtends[3528] = 76;
	fExtends[3529] = 72;
	fExtends[3530] = 89;
	fExtends[3531] = 104;
	fExtends[3532] = 108;
	fExtends[3533] = 89;
	fExtends[3534] = 130;
	fExtends[3535] = 87;
	fExtends[3536] = 81;
	fExtends[3537] = 81;
	fExtends[3538] = 86;
	fExtends[3539] = 85;
	fExtends[3540] = 81;
	fExtends[3541] = 66;
	fExtends[3542] = 61;
	fExtends[3543] = 69;
	fExtends[3544] = 39;
	fExtends[3545] = 39;
	fExtends[3546] = 92;
	fExtends[3547] = 99;
	fExtends[3548] = 106;
	fExtends[3549] = 118;
	fExtends[3550] = 119;
	fExtends[3551] = 140;
	fExtends[3552] = 166;
	fExtends[3553] = 116;
	fExtends[3554] = 118;
	fExtends[3555] = 119;
	fExtends[3556] = 124;
	fExtends[3557] = 57;
	fExtends[3558] = 63;
	fExtends[3559] = 58;
	fExtends[3560] = 59;
	fExtends[3561] = 115;
	fExtends[3562] = 92;
	fExtends[3563] = 121;
	fExtends[3564] = 85;
	fExtends[3565] = 81;
	fExtends[3566] = 34;
	fExtends[3567] = 38;
	fExtends[3568] = 115;
	fExtends[3569] = 115;
	fExtends[3570] = 59;
	fExtends[3571] = 59;
	fExtends[3572] = 59;
	fExtends[3573] = 59;
	fExtends[3574] = 59;
	fExtends[3575] = 59;
	fExtends[3576] = 59;
	fExtends[3577] = 59;
	fExtends[3578] = 59;
	fExtends[3579] = 59;
	fExtends[3580] = 59;
	fExtends[3581] = 59;
	fExtends[3582] = 59;
	fExtends[3583] = 59;
	fExtends[3584] = 100;
	fExtends[3585] = 100;
	fExtends[3586] = 100;
	fExtends[3587] = 100;
	fExtends[3588] = 100;
	fExtends[3589] = 100;
	fExtends[3590] = 100;
	fExtends[3591] = 100;
	fExtends[3592] = 100;
	fExtends[3593] = 100;
	fExtends[3594] = 100;
	fExtends[3595] = 100;
	fExtends[3596] = 100;
	fExtends[3597] = 100;
	fExtends[3598] = 100;
	fExtends[3599] = 100;
	fExtends[3600] = 100;
	fExtends[3601] = 100;
	fExtends[3602] = 100;
	fExtends[3603] = 100;
	fExtends[3604] = 100;
	fExtends[3605] = 100;
	fExtends[3606] = 100;
	fExtends[3607] = 100;
	fExtends[3608] = 0;
	fExtends[3609] = 0;
	fExtends[3610] = 0;
	fExtends[3611] = 0;
	fExtends[3612] = 0;
	fExtends[3613] = 100;
	fExtends[3614] = 100;
	fExtends[3615] = 100;
	fExtends[3616] = 100;
	fExtends[3617] = 100;
	fExtends[3618] = 100;
	fExtends[3619] = 100;
	fExtends[3620] = 100;
	fExtends[3621] = 100;
	fExtends[3622] = 100;
	fExtends[3623] = 100;
	fExtends[3624] = 100;
	fExtends[3625] = 100;
	fExtends[3626] = 100;
	fExtends[3627] = 100;
	fExtends[3628] = 200;
	fExtends[3629] = 200;
	fExtends[3630] = 200;
	fExtends[3631] = 200;
	fExtends[3632] = 300;
	fExtends[3633] = 300;
	fExtends[3634] = 300;
	fExtends[3635] = 300;
	fExtends[3636] = 400;
	fExtends[3637] = 400;
	fExtends[3638] = 400;
	fExtends[3639] = 400;
	fExtends[3640] = 500;
	fExtends[3641] = 500;
	fExtends[3642] = 500;
	fExtends[3643] = 500;
	fExtends[3644] = 600;
	fExtends[3645] = 600;
	fExtends[3646] = 600;
	fExtends[3647] = 600;
}

#else

void MusicFont::initialize()
{
	fRefSize = kFontSize;			// the font size used to compute the extends map
	fHeight	 = kGuidoFontHeight;	// the static height value

	fExtends[0] = 0;
	fExtends[1] = 44;
	fExtends[2] = 44;
	fExtends[3] = 100;
	fExtends[4] = 100;
	fExtends[5] = 100;
	fExtends[6] = 100;
	fExtends[7] = 100;
	fExtends[8] = 100;
	fExtends[9] = 80;
	fExtends[10] = 0;
	fExtends[11] = 100;
	fExtends[12] = 0;
	fExtends[13] = 0;
	fExtends[14] = 100;
	fExtends[15] = 100;
	fExtends[16] = 100;
	fExtends[17] = 100;
	fExtends[18] = 100;
	fExtends[19] = 100;
	fExtends[20] = 100;
	fExtends[21] = 100;
	fExtends[22] = 100;
	fExtends[23] = 100;
	fExtends[24] = 100;
	fExtends[25] = 100;
	fExtends[26] = 100;
	fExtends[27] = 100;
	fExtends[28] = 100;
	fExtends[29] = 100;
	fExtends[30] = 100;
	fExtends[31] = 100;
	fExtends[32] = 20;
	fExtends[33] = 42;
	fExtends[34] = 28;
	fExtends[35] = 42;
	fExtends[36] = 28;
	fExtends[37] = 103;
	fExtends[38] = 122;
	fExtends[39] = 188;
	fExtends[40] = 33;
	fExtends[41] = 33;
	fExtends[42] = 60;
	fExtends[43] = 60;
	fExtends[44] = 36;
	fExtends[45] = 60;
	fExtends[46] = 19;
	fExtends[47] = 108;
	fExtends[48] = 72;
	fExtends[49] = 50;
	fExtends[50] = 68;
	fExtends[51] = 64;
	fExtends[52] = 65;
	fExtends[53] = 60;
	fExtends[54] = 66;
	fExtends[55] = 68;
	fExtends[56] = 65;
	fExtends[57] = 66;
	fExtends[58] = 71;
	fExtends[59] = 83;
	fExtends[60] = 40;
	fExtends[61] = 200;
	fExtends[62] = 76;
	fExtends[63] = 136;
	fExtends[64] = 65;
	fExtends[65] = 30;
	fExtends[66] = 134;
	fExtends[67] = 84;
	fExtends[68] = 62;
	fExtends[69] = 60;
	fExtends[70] = 150;
	fExtends[71] = 40;
	fExtends[72] = 79;
	fExtends[73] = 40;
	fExtends[74] = 54;
	fExtends[75] = 56;
	fExtends[76] = 81;
	fExtends[77] = 117;
	fExtends[78] = 20;
	fExtends[79] = 71;
	fExtends[80] = 147;
	fExtends[81] = 60;
	fExtends[82] = 60;
	fExtends[83] = 106;
	fExtends[84] = 104;
	fExtends[85] = 111;
	fExtends[86] = 52;
	fExtends[87] = 128;
	fExtends[88] = 60;
	fExtends[89] = 200;
	fExtends[90] = 116;
	fExtends[91] = 51;
	fExtends[92] = 52;
	fExtends[93] = 89;
	fExtends[94] = 52;
	fExtends[95] = 100;
	fExtends[96] = 122;
	fExtends[97] = 50;
	fExtends[98] = 39;
	fExtends[99] = 84;
	fExtends[100] = 149;
	fExtends[101] = 4;
	fExtends[102] = 89;
	fExtends[103] = 31;
	fExtends[104] = 5;
	fExtends[105] = 30;
	fExtends[106] = 109;
	fExtends[107] = 110;
	fExtends[108] = 91;
	fExtends[109] = 121;
	fExtends[110] = 46;
	fExtends[111] = 28;
	fExtends[112] = 77;
	fExtends[113] = 60;
	fExtends[114] = 111;
	fExtends[115] = 240;
	fExtends[116] = 51;
	fExtends[117] = 112;
	fExtends[118] = 52;
	fExtends[119] = 80;
	fExtends[120] = 60;
	fExtends[121] = 149;
	fExtends[122] = 52;
	fExtends[123] = 21;
	fExtends[124] = 64;
	fExtends[125] = 91;
	fExtends[126] = 69;
	fExtends[127] = 100;
	fExtends[128] = 100;
	fExtends[129] = 100;
	fExtends[130] = 100;
	fExtends[131] = 100;
	fExtends[132] = 100;
	fExtends[133] = 100;
	fExtends[134] = 100;
	fExtends[135] = 100;
	fExtends[136] = 100;
	fExtends[137] = 100;
	fExtends[138] = 100;
	fExtends[139] = 100;
	fExtends[140] = 100;
	fExtends[141] = 100;
	fExtends[142] = 100;
	fExtends[143] = 100;
	fExtends[144] = 100;
	fExtends[145] = 100;
	fExtends[146] = 100;
	fExtends[147] = 100;
	fExtends[148] = 100;
	fExtends[149] = 100;
	fExtends[150] = 100;
	fExtends[151] = 100;
	fExtends[152] = 100;
	fExtends[153] = 100;
	fExtends[154] = 100;
	fExtends[155] = 100;
	fExtends[156] = 0;
	fExtends[157] = 100;
	fExtends[158] = 100;
	fExtends[159] = 100;
	fExtends[160] = 62;
	fExtends[161] = 39;
	fExtends[162] = 49;
	fExtends[163] = 48;
	fExtends[164] = 240;
	fExtends[165] = 51;
	fExtends[166] = 46;
	fExtends[167] = 41;
	fExtends[168] = 55;
	fExtends[169] = 35;
	fExtends[170] = 43;
	fExtends[171] = 24;
	fExtends[172] = 81;
	fExtends[173] = 0;
	fExtends[174] = 78;
	fExtends[175] = 4;
	fExtends[176] = 158;
	fExtends[177] = 122;
	fExtends[178] = 22;
	fExtends[179] = 55;
	fExtends[180] = 53;
	fExtends[181] = 170;
	fExtends[182] = 46;
	fExtends[183] = 91;
	fExtends[184] = 105;
	fExtends[185] = 105;
	fExtends[186] = 46;
	fExtends[187] = 170;
	fExtends[188] = 68;
	fExtends[189] = 121;
	fExtends[190] = 113;
	fExtends[191] = 60;
	fExtends[192] = 119;
	fExtends[193] = 138;
	fExtends[194] = 147;
	fExtends[195] = 288;
	fExtends[196] = 33;
	fExtends[197] = 33;
	fExtends[198] = 80;
	fExtends[199] = 54;
	fExtends[200] = 288;
	fExtends[201] = 60;
	fExtends[202] = 109;
	fExtends[203] = 56;
	fExtends[204] = 62;
	fExtends[205] = 39;
	fExtends[206] = 88;
	fExtends[207] = 6;
	fExtends[208] = 6;
	fExtends[209] = 94;
	fExtends[210] = 134;
	fExtends[211] = 113;
	fExtends[212] = 154;
	fExtends[213] = 123;
	fExtends[214] = 60;
	fExtends[215] = 60;
	fExtends[216] = 118;
	fExtends[217] = 60;
	fExtends[218] = 5;
	fExtends[219] = 60;
	fExtends[220] = 60;
	fExtends[221] = 109;
	fExtends[222] = 56;
	fExtends[223] = 149;
	fExtends[224] = 20;
	fExtends[225] = 34;
	fExtends[226] = 34;
	fExtends[227] = 86;
	fExtends[228] = 20;
	fExtends[229] = 104;
	fExtends[230] = 88;
	fExtends[231] = 62;
	fExtends[232] = 60;
	fExtends[233] = 60;
	fExtends[234] = 60;
	fExtends[235] = 60;
	fExtends[236] = 60;
	fExtends[237] = 60;
	fExtends[238] = 33;
	fExtends[239] = 33;
	fExtends[240] = 45;
	fExtends[241] = 45;
	fExtends[242] = 32;
	fExtends[243] = 100;
	fExtends[244] = 100;
	fExtends[245] = 100;
	fExtends[246] = 100;
	fExtends[247] = 100;
	fExtends[248] = 100;
	fExtends[249] = 100;
	fExtends[250] = 100;
	fExtends[251] = 100;
	fExtends[252] = 100;
	fExtends[253] = 100;
	fExtends[254] = 100;
	fExtends[255] = 100;
	fExtends[256] = 138;
}

#endif

